<?php
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
require_once 'SearchSettings.php';
require_once SearchSettings::ZEND_PATH;
require_once 'TextParser.php';

class SearchContext {
	
	const XML_AMP = '&amp;';
	const XML_APOS = '&apos;';
	const XML_CR = '&#13;';
	const XML_GT = '&gt;';
	const XML_LT = '&lt;';
	const XML_LF = '&#10;';
	const XML_QUOT = '&quot;';
	const XML_TAB = '&#9;';
	const XML_DECLARATION = '<?xml version="1.0" encoding="UTF-8"?>';
	
	protected static $keyWords = array('Product', 'Descriptor', 'Process', 'Pattern',
							'Type', 'Definition', 'Set', 'Asset', 'Considerations',  
							'Category', 'Material', 'Grouping', 'Tool');
	
	protected $indexPath;
	protected $hitsPerPage;
	protected $searchString;
	protected $xsl;
	protected $docBase;
	protected $docFolder;
	protected $searchResults;
	protected $hitsCount;
	protected $phpPageName;
	protected $currentPage;
	protected $dispatherPage;
	protected $isCJKAnalyzer;

	public function __construct($dispatherPage, $searchString, $hitsPerPage)
	{
		if (isset($dispatherPage))
		{
			$this->dispatherPage = $dispatherPage;
		}
		$this->searchString = $searchString;
		$this->hitsPerPage = $hitsPerPage;
		
		$this->_initialize();
		$this->_doSearch();
	}
	
	private function _initialize()
	{
		$this->indexPath = realpath('./index/');
		$this->xsl = realpath('./search_result.xsl');
		$version = file($this->indexPath."/version.txt");
		if (preg_match('/^CJKAnalyzer$/u', isset($version[1]) ? $version[1] : null))
		{
			$this->isCJKAnalyzer = true;
		}
		else
		{
			$this->isCJKAnalyzer = false;
		}
		$this->docFolder = explode('*',$version[0]);
		$this->docFolder = $this->docFolder[0];
		
		$protocol = 'http://';
		if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off'|| $_SERVER['SERVER_PORT'] == 443) {
		    $protocol = 'https://';
		}
		$this->phpPageName = $protocol.$_SERVER['SERVER_NAME'].':'.$_SERVER["SERVER_PORT"].$_SERVER['PHP_SELF'];
		
		$this->docBase = substr($this->phpPageName, 0, strrpos($this->phpPageName, '/'));
		$this->docBase = substr($this->docBase, 0, strrpos($this->docBase, '/'));
	}
	
	private function _doSearch()
	{
		$this->currentPage = -1;
		if ($this->_isInvalidSearchString($this->searchString))
		{
			$this->hitsCount = 0;
			return;
		}
		Zend_Search_Lucene_Analysis_Analyzer::setDefault(new TextAnalyzer($this->isCJKAnalyzer));
		Zend_Search_Lucene::setDefaultSearchField('contents');
		$index = Zend_Search_Lucene::open($this->indexPath);
		$query = TextParser::parse($this->searchString, "UTF-8");
		$hits = $index->find($query);
		$this->hitsCount = count($hits);
		for ($i = 0; $i < $this->hitsCount; $i++)
		{
			$hit = $hits[$i];
			$document = $hit->getDocument();
			$url =  $document->getFieldValue('url');
			$title = $document->getFieldValue('title');
			$summary = $document->getFieldValue('summary');;
			$umaType = $document->getFieldValue('uma.type');
			if(strcasecmp($umaType, "udt")==0){
				$searchicon=$document->getFieldValue('searchicon');
				$this->searchResults[$i]['searchicon']=$searchicon;
			}
			$this->searchResults[$i]['url'] = $url;
			$this->searchResults[$i]['title'] = $title;
			$this->searchResults[$i]['summary'] = $summary;
			$this->searchResults[$i]['uma.type'] = $umaType;
		}
	}
	
	public function getPrevPage()
	{
		$this->currentPage--;
		$result = $this->_getResults();
		return $result;
	}
	
	public function getNextPage()
	{
		$this->currentPage++;
		$result = $this->_getResults();
		return $result;
	}
	
	private function _getResults()
	{
		try {
			$initPageNumber = $this->currentPage * $this->hitsPerPage;
			$maxPageNumber = min($this->hitsCount, $initPageNumber + $this->hitsPerPage);
			$nextEnabled = ($this->hitsCount > ($initPageNumber + $this->hitsPerPage)) ? 'true' : 'false';
			$prevEnabled = ($this->currentPage > 0) ? 'true' : 'false';		
			$searchResultXML =  self::XML_DECLARATION.'<SearchResult'.
								' totalHits="'.$this->hitsCount.
								'" initPageNum="'.(($initPageNumber > 0) ? $initPageNumber + 1 : (($this->hitsCount < 1) ? 0 : 1)).
								'" maxPageNum="'.$maxPageNumber.
								'" hitsPerPage="'.$this->hitsPerPage.
								'" nextEnabled="'.$nextEnabled.
								'" prevEnabled="'.$prevEnabled.
								'" nextText="'.SearchSettings::NEXT_ACTION_TEXT.
								'" prevText="'.SearchSettings::PREVIOUS_ACTION_TEXT.
								'" imagePath="'.$this->docBase.'/search/images/'.
								'" searchString="'.TextParser::_filterCharacter($this->searchString).
								'" nextHref="'.$this->dispatherPage.'?action=next'.
								'" prevHref="'.$this->dispatherPage.'?action=prev'.
								'">';
			
			for($i = $initPageNumber; $i < $maxPageNumber; $i++)
			{
				$url =  $this->searchResults[$i]['url'];
				$title = $this->searchResults[$i]['title'];
				$summary = $this->searchResults[$i]['summary'];
				$umaType = $this->searchResults[$i]['uma.type'];
				
				$searchResultXML.= '<SearchHit';
				if (isset($url)) 
				{
					$fixedUrl = $this->docBase.substr($url, strpos($url, $this->docFolder)+ strlen($this->docFolder));
					$searchResultXML.= ' url="'.$this->urlEscape($fixedUrl).'"';
				}
				
				if (isset($title)) 
				{
					//$searchResultXML.= ' title="'.$title.'"';
					$searchResultXML.= ' title="'.$this->htmlEscape($title).'"';
				}
				
				if (isset($summary)) 
				{
					$searchResultXML.= ' summary="'.$this->htmlEscape($summary).'"';
				}
				
				if (isset($umaType)) 
				{
					// Here will be modified later
					if (strcasecmp($umaType, 'General_content') == 0)
					{
						$searchResultXML.= ' icon="Other.gif"';
					}else if(strcasecmp($umaType, "udt")==0){
						$shapeImage=$this->searchResults[$i]['searchicon'];
						if(isset($shapeImage)){
							$searchIcon="./../../".$shapeImage;
							$searchResultXML.=' icon="'.$searchIcon.'"';
						}else{
							$searchResultXML.=' icon="Other.gif"';
						}
					}
					else
					{
						$searchResultXML.= ' icon="'.$this->convertFirstLetterCase($umaType, self::$keyWords).'.gif"';
					}				
				}
				$searchResultXML.= '/>';
			}
			$searchResultXML.= '</SearchResult>';

			//echo htmlentities($searchResultXML);
			
			//  XML transform
			$processor = new XSLTProcessor;
			$xsl = new DOMDocument;
			$xsl->load($this->xsl);
			$xml = new DOMDocument;
			$xml->loadXML($searchResultXML);
			$processor ->importStyleSheet($xsl);
			$result = $processor ->transformToXML($xml);
			// echo htmlentities($result);
		}  catch(Exception $e) {
			$result = SearchSettings::SEARCH_FAILED;
		}
		
		return $result;
	}
	
	protected function urlEscape($url)
	{
		$result = str_replace('\\', '/', $url);
		return $result;
	}
	
	protected function htmlEscape($content)
	{
		$result = "";
		$len = strlen($content);
		for ($i = 0; $i < $len; $i++) {
			$ch = $content[$i];
			switch ($ch) {
			case '<':
				$result .= self::XML_LT;
				break;
			case '>':
				$result .= self::XML_GT;
				break;
			case '&':
				$result .= self::XML_AMP;
				break;
			case '"':
				$result .= self::XML_QUOT;
				break;
			case '\'':
				$result .= self::XML_APOS;
				break;
			case '\r':
				$result .= self::XML_CR;
				break;
			case '\n':
				$result .= self::XML_LF;
				break;
			default:
				$result .= $ch;
				break;
			}
		}
		
		return $result;
	}
	
	protected function convertFirstLetterCase($str, $keyWords)
	{
		$lstr = ucfirst(strtolower($str));
		foreach($keyWords as $keyWord)
		{
			$lstr = str_ireplace($keyWord, $keyWord, $lstr);
		}
		return $lstr;
	}
	
	private function _isInvalidSearchString($str)
	{
		if (($str==null)||(trim($str)=='')||(iconv_strlen($str, 'UTF-8')==1))
		{
			return true;
		}
		return false;
	}
}

?>