package com.rational.rpw.compositetreeview;

import java.awt.Cursor;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.Autoscroll;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import com.rational.rpw.dnd.CustomDnDConstants;
import com.rational.rpw.dnd.DragGestureAdapter;
import com.rational.rpw.dnd.DragSourceAdapter;
import com.rational.rpw.dnd.DropTargetAdapter;


/**
 * This class is a generic tree view which can be used by
 * various applications.  It is a specialization of the
 * Java Swing JTree.  It provides drag and/or drop capability
 * if requested as well as the ability to detect and show
 * popup menus.
 *
 * Being a generic tree, it relies on user-provided tree cell
 * renderers to properly render tree nodes.
 */
public class CompositeTreeView extends JTree
    implements TreeChangeListener, Autoscroll
{
    // mouse listener for popup menu detection
    private PopupListener _popupListener = null;
    
    // popup menu adapter
    private TreePopupMenuAdapter _popupMenuAdapter = null;
    
    // drag component tree
    private TreeDragComponent _treeDragComponent = null;
    
    // drop component tree
    private TreeDropComponent _treeDropComponent = null;
    
    // drag source adapter
    private DragSourceAdapter _dragSourceAdapter = null;
    
    // drag gesture adapter
    private DragGestureAdapter _dragGestureAdapter = null;
    
    // drop target adapter
    private DropTargetAdapter _dropTargetAdapter = null;
    
    // drop target
    private DropTarget _dropTarget = null;
    
    /** 
     * Constructor.    
     * @parma   treeNode              The tree node to display in the tree.
     * @param   customCellRenderer    The custom tree cell renderer.
     */
    public CompositeTreeView( TreeNode treeNode,
                              TreeCellRenderer customCellRenderer )
    {
        super( treeNode );

        // set the renderer
        setCellRenderer( customCellRenderer );
        
        // don't show root, but show root handles
        setRootVisible( false );
        setShowsRootHandles( true );
        
        // set line style        
		putClientProperty( "JTree.lineStyle", "Angled" );
    }
    
    /**
     * Enable detection for popup menus.
     */
    public void enablePopup( TreePopupMenuAdapter adapter )
    {
        setPopupDetection( adapter );
    }
    
    /**
     * Disable detection for popup menus.
     */
    public void disablePopup()
    {
        setPopupDetection( null );
    }
    
    /**
     * By default the root node of the tree is not shown. Clients 
     * can change this behaviour by calling this method.     */
    public void setRootVisible()
    {
    	setRootVisible( true );	
    }
    
    /** 
     * Enable this tree as a drag source.
     * @param   dragAction      The permitted drag action for this tree.
     *                          One of the actions in java.awt.dnd.DnDConstants.
     */
    public void enableDragSource( int dragAction )
    {
        // set up the tree drag component
        // note that if the action is MOVE_NO_REMOVE, we have to
        // use the customized tree drag component
        TreeDragComponent dragComponent = null;
        if( dragAction == CustomDnDConstants.ACTION_MOVE_NO_REMOVE )
        {
            dragComponent = new TreeDragAsLinkComponent( this );
            // reset the drag action to MOVE because that is how
            // the default Java gesture will be recognized
            dragAction = DnDConstants.ACTION_MOVE;
        }
        else
        {
            dragComponent = new TreeDragComponent( dragAction, this );
        }        
        enableDragSource( dragAction, dragComponent );
    }
    
    /** 
     * Enable this tree as a drag source.  Allows user to specify
     * a custom tree drag component.
     * @param   dragAction      The permitted drag action for this tree.
     *                          One of the actions in java.awt.dnd.DnDConstants.
     * @param   dragComponent   The custom tree drag component, which is
     *                          a subclass of TreeDragComponent.
     * @see com.rational.rpw.compositetreeview.TreeDragComponent
     */
    public void enableDragSource( int dragAction,
                                  TreeDragComponent dragComponent )
    {
        if( dragAction == CustomDnDConstants.ACTION_MOVE_NO_REMOVE )
        {
            // reset the drag action to MOVE because that is how
            // the default Java gesture will be recognized
            dragAction = DnDConstants.ACTION_MOVE;
        }
        _treeDragComponent = dragComponent;
        
        // set up the drag source
    	DragSource dragSource = DragSource.getDefaultDragSource();
    	Cursor cursor = DragSource.DefaultMoveDrop;
    	if( dragAction == DnDConstants.ACTION_COPY )
    	{
    	    cursor = DragSource.DefaultCopyDrop;    
    	}
	    _dragGestureAdapter = new DragGestureAdapter( _treeDragComponent, cursor );
	    _dragSourceAdapter = new DragSourceAdapter( _treeDragComponent );
	    
	    // set up the tree drag component's drag source listener
	    // TBD fchong: I AM NOT HAPPY WITH THIS PART OF THE DESIGN
	    // AS THIS IS A CIRCULAR REFERENCE!!  FIX LATER.
	    _treeDragComponent.setDragSourceListener( _dragSourceAdapter );
    
	    // component, action, listener
	    dragSource.createDefaultDragGestureRecognizer( this,
	                                                   dragAction,
	                                                   _dragGestureAdapter ); 
    }
    
    /**
     * Enable this tree as a drop target.
     * @param   acceptableDropAction    acceptable drop action
     * @param   acceptableDropFlavors   list of acceptable drop flavors
     */
/*    public void enableDropTarget( int acceptableDropAction,
                                  DataFlavor[] acceptableDropFlavors )
    {
        enableDropTarget( acceptableDropAction,
                          acceptableDropFlavors,
                          new TreeDropComponent( this ) );
    }
 */   
    /**
     * Enable this tree as a drop target.
     * @param   acceptableDropAction    acceptable drop action
     * @param   acceptableDropFlavors   list of acceptable drop flavors
     * @param   dropComponents          custom drop component, subclass of
     *                                  TreeDropComponent
     * @see com.rational.rpw.compositetreeview.TreeDropComponent
     */
    public void enableDropTarget( int acceptableDropAction,
                                  DataFlavor[] acceptableDropFlavors,
                                  TreeDropComponent treeDropComponent )
    {
        // note that we need to account for customized action,
        // which the default Java gesture will treat as a move,
        // so we should set it as so
        if( acceptableDropAction == CustomDnDConstants.ACTION_MOVE_NO_REMOVE )
        {
            acceptableDropAction = DnDConstants.ACTION_MOVE;
        }
        
        // set up tree drop component
        _treeDropComponent = treeDropComponent;
        
	    // set up drop target adapter
	    _dropTargetAdapter = new DropTargetAdapter( _treeDropComponent, 
					                                acceptableDropAction,
					                                acceptableDropFlavors );

	    // set up drop target
	    _dropTarget = new DropTarget( this, 
					                  acceptableDropAction,
					                  _dropTargetAdapter,
					                  true );
	    _dropTarget.setActive( true );        
    }
    
    /**
     * Called by the user when the node has changed, the node is
     * repainted.  Implementation of ICompositeTreeActionListener interface.
     * @param   changedNode     The node which has changed.
     */
    public void nodeChanged( MutableTreeNode changedNode )
    {
        ((DefaultTreeModel)this.getModel()).nodeChanged( changedNode );   
    }
    
    /**
     * Called by the user when the node structure has changed, 
     * the node is reloaded and repainted.
     * Implementation of ICompositeTreeActionListener interface.
     * @param   changedNode     The node which has changed.
     */
    public void nodeStructureChanged( MutableTreeNode changedNode )
    {
       ((DefaultTreeModel)this.getModel()).nodeStructureChanged( changedNode );
    }

    /**
     * Called by the user when the selection has changed.
     * @param   nodeToSelect    The node to select.
     */
    public void nodeSelectionChanged( MutableTreeNode nodeToSelect )
    {
    	this.removeSelectionPath( this.getSelectionPath() );
    	DefaultMutableTreeNode node = (DefaultMutableTreeNode)nodeToSelect;
    	TreePath path = new TreePath( node.getPath() );
    	this.setSelectionPath(  path );
    }

    /**
     * Autoscroll method.  Called repeatedly while dragging.
     */
    public Insets getAutoscrollInsets() 
    {
	    int margin = 50;
    	Rectangle bounds = getVisibleRect();    
	    Insets si = new Insets( bounds.y + margin,
		            	        bounds.x  + margin,
			                    bounds.height - margin,
			                    bounds.width - margin );
	    return si;
    }

    /**
     * Autoscroll method.  Called repeatedly while dragging.   
     */
    public void autoscroll( Point p ) 
    {
	    int crow = getClosestRowForLocation( p.x, p.y );
	    Rectangle rbounds = getRowBounds( crow );

	    TreePath cpath = getClosestPathForLocation( p.x, p.y );
	    Rectangle pbounds = getPathBounds( cpath );

	    Rectangle bounds = rbounds;
	    int size = 30;
	    if( null != bounds )
	    size=(int)bounds.getHeight();
	    int hsize = size/2; 
    	Rectangle rect = new Rectangle( p.x-hsize, p.y-hsize, size, size );
	    scrollRectToVisible( rect );
    }
    
        
    /**
     * Add or remove mouselistener depending on popup detection state.
     */
    private void setPopupDetection( TreePopupMenuAdapter popupMenuAdapter )
    {
        // add mouse listener if requested
        if( null != popupMenuAdapter )
        {
            // set up the menu adapter
            _popupMenuAdapter = popupMenuAdapter;
            _popupMenuAdapter.setTreeChangeListener( this );
            
            if( null == _popupListener )
            {
                _popupListener = new PopupListener();   
            }
            addMouseListener( _popupListener );
        }
        else
        {
            _popupMenuAdapter = null;
            if( null != _popupListener )
            {
                removeMouseListener( _popupListener );
            }
            _popupListener = null;
        }
    }
    
    /**
     * Inner class to detect mouse actions, for popup menus.
     */
    class PopupListener extends MouseAdapter
    {
        public PopupListener()
        {
        }
        
        public void mousePressed( MouseEvent e ) 
        {
            maybeShowPopup(e);
        }

        public void mouseReleased( MouseEvent e ) 
        {
            maybeShowPopup(e);
        }

        private void maybeShowPopup( MouseEvent e ) 
        {
            if(e.isPopupTrigger() ) 
            {
            	// first see if this is a multiple selection
            	TreePath[] multiPaths = ( (CompositeTreeView)e.getComponent() ).getSelectionPaths();
            	if( null != multiPaths && multiPaths.length > 1 )
            	{
            		Vector multiNodes = new Vector();
            		for( int i = 0; i < multiPaths.length; i++ )
            		{
	 	            	TreePath selPath = multiPaths[ i ];         
		              	if( null != selPath )
  		              	{
    	            		MutableTreeNode node = (MutableTreeNode)selPath.getLastPathComponent();
							multiNodes.addElement( node );
 	          	     	}
            		}
 	                if( _popupMenuAdapter.isMenuAvailable( multiNodes ) )
 	               	{
  	                	JPopupMenu popup = _popupMenuAdapter.getMenu( multiNodes );
                   		popup.show( e.getComponent(),
                  		            e.getX(), 
                   		            e.getY());
               		}           		
            	}
            	else
            	{
            		// only one node or no node specifically selected
 	            	TreePath selPath = ( (CompositeTreeView)e.getComponent() ).getPathForLocation( e.getX(), e.getY() );                
	              	if( null != selPath )
                	{
                		MutableTreeNode node = (MutableTreeNode)selPath.getLastPathComponent();
	                	if( _popupMenuAdapter.isMenuAvailable( node ) )
 	               		{
  	                		JPopupMenu popup = _popupMenuAdapter.getMenu( node );
                    		popup.show( e.getComponent(),
                     		            e.getX(), 
                       		            e.getY());
 	               		}
 	               	}
                }
            }
        }
    }       
    
}