package com.rational.rpw.compositetreeview;

import java.awt.Point;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragSourceListener;
import java.io.Serializable;
import java.util.Vector;

import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;

import com.rational.rpw.dnd.IDragComponent;
import com.rational.rpw.dnd.datatransfer.TreeTransferable;

/**
 * This class represents a tree as a drag source.  It implements
 * the IDragComponent interface, so that is can be used as
 * a drag source by a DragSourceAdapter. <br>
 * CompositeTreeView classes should contain this class and
 * enable it if drag source behaviour is requested.  This class
 * maintains a reference to the containing tree.
 */
public class TreeDragComponent implements IDragComponent, Serializable
{
    // default drag action
    protected int _dragAction = DnDConstants.ACTION_COPY_OR_MOVE;
    
    // reference to container tree
    protected JTree _tree = null;
    
    // reference to list of nodes to move
    protected Vector _nodesToMove = null;
    
    // reference to drag source listener
    protected DragSourceListener _dragSourceListener = null;
    
    /**
     * Constructor.
     * @param   dragAction  The drag action for this adapter.
     *                      <b>Must be one of the actions in DnDConstants.</b>
     * @param   tree        The container tree.
     * @see java.awt.dnd.DnDConstants
     */
    public TreeDragComponent( int dragAction,
                              JTree tree )
    {
        _dragAction = dragAction;  
        _tree = tree;
    }

    /**
     * Called by the DragSourceAdapter in dragDropEnd if the operation
     * is DnDConstants.ACTION_MOVE 
     */
    public void move()
    {
        MutableTreeNode singleNodeToMove = null;
        MutableTreeNode parent = null;
        for( int i = 0; i < _nodesToMove.size(); i++ )
        {
            singleNodeToMove= ( (MutableTreeNode)_nodesToMove.elementAt( i ) );

    	    // do the move
	        DefaultTreeModel model = (DefaultTreeModel)_tree.getModel();
	        parent = (MutableTreeNode)singleNodeToMove.getParent();
	        if( null != parent )
	        {
	        	parent.remove( singleNodeToMove );
	        }
	        model.nodeStructureChanged( parent );
        }
    }

    /**
     * Returns the drag action for this adapter.
     * @return  int     The drag action for this adapter.
     */
    public int getDragAction()
    {
        return( _dragAction );
    }

    /**
     * Called by the DragGestureAdapter in dragGestureRecognized
     * @return  Transferable    The transferable object at the given point.
     */
    public Transferable getTransferable( Point point )
    {
	    Vector nodes = getTreeNodes( point );
	    TreeTransferable transferable = new TreeTransferable( nodes );

        // keep store of nodes to move
	    _nodesToMove = nodes;
	    
	    return( transferable );
    }

    /**
     * Called by the DragGestureAdapter in dragGestureRecognized. This
     * is the listener that is registered with the drag source in the
     * startDrag method.
     *@returns usually the DragSourceAdapter 
     */
    public DragSourceListener getDragSourceListener()
    {
        return( _dragSourceListener );
    }

    /**
     * Sets the drag source listener.
     */
    public void setDragSourceListener( DragSourceListener listener )
    {
        _dragSourceListener = listener;
    }

    /**
     * Is the location within the component draggable?  Called by the
     * DragGestureAdapter in dragGestureRecognized. If false is
     * returned then there will be no drag initiated.  
     */
    public boolean isStartDragOk( DragGestureEvent dragGestureEsvent )
    {
        return( true );
    }
    
    /**
     * Returns the selected tree nodes.
     */
    protected Vector getTreeNodes( Point p ) 
    {
	    TreePath paths[] = _tree.getSelectionPaths();
        Vector nodeViewList = new Vector();
	    
	    if( null != paths && paths.length > 0 )
	    {
	        Object node = null;
	        MutableTreeNode nodeView = null;
	        for( int i = 0; i < paths.length; i++ )
	        {
	            node = ( (TreePath)paths[i] ).getLastPathComponent();
	            nodeView = (MutableTreeNode)node;   
	            nodeViewList.add( nodeView );
	        }
	    } 
	    return( nodeViewList );
    }

}