package com.rational.rpw.compositetreeview;

import java.awt.Color;
import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.io.File;
import java.io.Serializable;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import com.rational.rpw.dnd.IDropComponent;
import com.rational.rpw.dnd.datatransfer.FileListTransferable;
import com.rational.rpw.dnd.datatransfer.TreeTransferable;


/**
 * This class represents a tree as a drop target.  It implements
 * the IDropComponent interface, so that is can be used as
 * a drop target by a DropTargetAdapter. <br>
 * CompositeTreeView classes should contain this class and
 * enable it if drop target behaviour is requested.  This class
 * maintains a reference to the containing tree.
 */
public class TreeDropComponent
    implements IDropComponent, Serializable
{
    // reference to container tree
    protected CompositeTreeView _tree = null;

    // default background selection color
    protected Color _background = null;

    /**
     * Constructor.
     * @param   tree        The container tree.
     * @see java.awt.dnd.DnDConstants
     */
    public TreeDropComponent( CompositeTreeView tree )
    {
        _tree = tree;
        _background = ( (DefaultTreeCellRenderer)_tree.getCellRenderer() ).getBackgroundSelectionColor();
    }

    /**
     * Called by the DropTargetAdapter in dragEnter, dragOver and
     * dragActionChanged
     */
    public void dragUnderFeedback( boolean ok,
                                   DropTargetDragEvent dropTargetDragEvent )
    {
	    if( !ok )
	    {
	        // create border
	        DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer)_tree.getCellRenderer();
	        renderer.setBorderSelectionColor( Color.black );
	        renderer.setBackgroundSelectionColor( Color.red );
	    }
	    else
	    {
	        // normal color
	        DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer)_tree.getCellRenderer();
	        renderer.setBorderSelectionColor( null );
	        renderer.setBackgroundSelectionColor( _background );
	    }

	    Point p = dropTargetDragEvent.getLocation();
	    TreePath path = _tree.getPathForLocation( p.x, p.y );
	    if( null != path )
	    {
	        // select the path
	        _tree.setSelectionPath( path );
	        if( !_tree.isExpanded( path ) )
	        {
	            // expand path if not already expanded
                // but only if drag is ok
                if( ok )
                {
					Timer timer = new Timer();
        			timer.schedule( new ExpandNodeTask( path, timer ), 1500 );        			
                }
	        }
	    }
	    else
	    {
	    	// deselect any path
	    	_tree.removeSelectionPath( _tree.getSelectionPath() );
	    }
    }

    /**
     * Returns <code>true</code> if the node allows children.
     * <b>Must be overriden if more rules are needed to determine permission.</b>
     */
    public boolean isDragOk( DropTargetDragEvent dropTargetDragEvent )
    {
	    /*Point p = dropTargetDragEvent.getLocation();
	    TreePath path = _tree.getPathForLocation( p.x, p.y );
	    TreeNode receivingNode = null;
	    if( null != path )
	    {
	        receivingNode = (TreeNode)path.getLastPathComponent();
	    }
	    if( null != receivingNode && receivingNode.getAllowsChildren() )
	    {
	        return true;
	    }
	    return false;*/
	    
	    // drag permitted anywhere on tree, not just on nodes
	    return( true );
    }

    /**
     * Called by the DropTargetAdapter in dragExit and drop
     */
    public void undoDragUnderFeedback()
    {
        // undo selection and color
        _tree.clearSelection();
    	( (DefaultTreeCellRenderer)_tree.getCellRenderer() )
    	    .setBackgroundSelectionColor( _background );    	
    }

    /**
     * Called by the DropTargetAdapter in drop
     */
    public boolean add( Object objectToDrop,
                        DropTargetDropEvent dropTargetDropEvent,
                        DataFlavor dataFlavor )
    {
        boolean status = false;

        // row for add
        TreePath path = null;
        Point p = dropTargetDropEvent.getLocation();
        int row =  _tree.getRowForLocation( p.x, p.y );
        if( -1 == row )
        {
            // get path to root of tree
            TreeNode root = (TreeNode)_tree.getModel().getRoot();
            path = new TreePath( root );
    	}
		else
		{
	    	// parent node and set row back to 0, so node can be inserted at beginning
        	path = _tree.getClosestPathForLocation( p.x, p.y );
		}

        MutableTreeNode	nodeToContain = (MutableTreeNode)path.getLastPathComponent();
        row = nodeToContain.getChildCount();

		if( dataFlavor.equals( TreeTransferable._treeNodeFlavor ) )
		{
	    	// do the add
	    	// for a tree, the object to drop is a Vector of tree nodes
	    	status = addTreeNodes( dropTargetDropEvent, (Vector)objectToDrop, nodeToContain, row );
		}
        else if( dataFlavor.equals( FileListTransferable._fileListFlavor ) )
        {
            Vector fileList = new Vector();
                        
            // do the add
	        // for a file list, the object to drop is a Vector of files
            if( objectToDrop instanceof Vector )
            {
                fileList = (Vector)objectToDrop;
            }
            else
            {
                // this likely came from a component like Windows Explorer,
                // treat it like a list
                java.util.List list = (java.util.List)objectToDrop;
                Iterator iterator = list.iterator();
                while (iterator.hasNext())
                {
                    File file = (File)iterator.next();
                    fileList.addElement( file );
                }
            }

            status = addFileList( fileList, nodeToContain, row );
        }

		return( status );
    }

    /**
     * Adds the childnode (i.e. node to drop) into the row in the tree.
     */
    protected boolean addTreeNodes( DropTargetDropEvent dropTargetDropEvent,
                                    Vector objectToDrop,
                                    MutableTreeNode parent,
                                    int row )
    {
        int dropAction = dropTargetDropEvent.getDropAction();

        MutableTreeNode nodeToDrop = null;
        for( int i = 0; i < objectToDrop.size(); i++ )
        {
            nodeToDrop = (MutableTreeNode)objectToDrop.elementAt( i );

            if( parent.equals( nodeToDrop ) )
            {
		        // same node, cannot drop
		        _tree.getToolkit().beep();
		        return( false );
            }
            if( parent.getParent() != null &&
                parent.getParent().equals( nodeToDrop ) )
            {
                // descendant conflict, cannot drop
	        	_tree.getToolkit().beep();
	        	return( false );
        	}
        	if( nodeToDrop.getParent().equals( parent ) )
        	{
                // node already in parent
	        	_tree.getToolkit().beep();
	        	return( false );        		
        	}
	    	if( parent.getAllowsChildren() )
	    	{
	        	// clone the node
            	MutableTreeNode newNode = cloneTreeNode( nodeToDrop );

	  	        // do the insert
	        	DefaultTreeModel model = (DefaultTreeModel)_tree.getModel();
        		model.insertNodeInto( newNode, 
        		                      parent,
        		                      row );
        	}
  	    	else
	    	{
		        // no children allowed, cannot drop
		        _tree.getToolkit().beep();
	 	        return( false );
	    	}
        }

	    return( true );
    }

    /**
     * Clones node.
     */
    protected MutableTreeNode cloneTreeNode( MutableTreeNode inputNode )
    {
        DefaultMutableTreeNode outputNode = null;

        // clone the node
        outputNode = new DefaultMutableTreeNode( inputNode );

        // now clone children
		int nkids = inputNode.getChildCount();
		for( int j = 0; j < nkids; j++ )
    	{
	    	MutableTreeNode node = (MutableTreeNode)inputNode.getChildAt( j );
            // recursive clone
            outputNode.insert( cloneTreeNode( node ), j );
		}

    	return( outputNode );
    }

    /**
     * Adds the file list into the row in the tree.
     */
    protected boolean addFileList( Vector objectToDrop,
                                   MutableTreeNode parent,
                                   int row )
    {
        boolean status = false;

        DefaultMutableTreeNode nodeToDrop = null;
        for( int i = 0; i < objectToDrop.size(); i++ )
        {
            nodeToDrop = new DefaultMutableTreeNode( (File)objectToDrop.elementAt( i ) );
            if( parent.equals( nodeToDrop ) )
            {
		        // same node, cannot drop
		        _tree.getToolkit().beep();
		        return( false );
            }
            if( parent.getParent() != null &&
                parent.getParent().equals( nodeToDrop ) )
            {
                // descendant conflict, cannot drop
	        	_tree.getToolkit().beep();
	        	return( false );
        	}
        	if( nodeToDrop.getParent() != null &&
        	    nodeToDrop.getParent().equals( parent ) )
        	{
                // node already in parent
	        	_tree.getToolkit().beep();
	        	return( false );        		
        	}
            if( parent.getAllowsChildren() )
            {
    	        // do the insert
	            DefaultTreeModel model = (DefaultTreeModel)_tree.getModel();
	            model.insertNodeInto( nodeToDrop,
	                                  parent,
	                                  row );
                status = true;
            }
    	    else
	        {
	            // no children allowed, cannot drop
	            _tree.getToolkit().beep();
	            return( false );
	        }
        }

	    return( status );
    }

	/**
	 * Simple thread to expand tree node.
	 */
	protected class ExpandNodeTask extends TimerTask 
	{
		private TreePath _path = null;
		private Timer _timer = null;
		
		public ExpandNodeTask( TreePath path,
		                       Timer timer )
		{
			super();
			_path = path;	
			_timer = timer;
		}
		
        public void run() 
        {
        	if( null != _tree &&
        	    null != _path &&
        	    null != _tree.getSelectionPath() &&
        	    _tree.getSelectionPath().equals( _path ) )
        	{
            	_tree.expandPath( _path );	            
        	}
            _timer.cancel();
            _timer = null;
            this.cancel();
        }
    }


}