package com.rational.rpw.dnd;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.io.*;
import java.util.*;
import java.util.List;

/**
 * This class implements the DropTargetListener interface.
 * It acts as a listener that will start the drag operation.
 * It maintains a reference to an IDropComponent which will be
 * queried for the drag and feedback actions.
 */
public class DropTargetAdapter implements DropTargetListener, Serializable
{
    protected DataFlavor[] _acceptableDropFlavors = null;
    protected int _acceptableDropAction = DnDConstants.ACTION_MOVE;
    protected IDropComponent _dropComponent = null;
  
    /**
     * Constructor.
     */
    public DropTargetAdapter( IDropComponent dropComponent, 
			                  int            acceptableDropAction,
			                  DataFlavor[]   acceptableDropFlavors ) 
	{
	    _dropComponent = dropComponent;
    	_acceptableDropFlavors = acceptableDropFlavors;
	    _acceptableDropAction = acceptableDropAction;
    }
    
    // DropTargetListener methods
    /**
     * start "drag under" feedback on component
     * invoke acceptDrag or rejectDrag based on isDragOk
     */
    public void dragEnter( DropTargetDragEvent e ) 
    {
	    if( !isDragOk( e ) ) 
	    {
    	    // drop not permitted
    	    rejectDrag( e );
	        return;
	    }
	    _dropComponent.dragUnderFeedback( true, e );
	    e.acceptDrag( e.getDropAction() );      
    }

    /**
     * continue "drag under" feedback on component
     * invoke acceptDrag or rejectDrag based on isDragOk
     */
    public void dragOver( DropTargetDragEvent e ) 
    {
    	if( !isDragOk(e) ) 
    	{
    	    // drop not permitted
    	    rejectDrag( e );
    	    return;
	    }
	    _dropComponent.dragUnderFeedback( true, e );
	    e.acceptDrag( e.getDropAction() );            
    }
    
    /**
     * called when user has modified current drop gesture
     */
    public void dropActionChanged( DropTargetDragEvent e ) 
    {
    	if( !isDragOk(e) ) 
    	{
    	    // drop not permitted
    	    rejectDrag( e );
    	    return;
	    }
    	_dropComponent.dragUnderFeedback( true, e );
	    e.acceptDrag( e.getDropAction() );            
    }
    
    /**
     * Called while a drag operation is ongoing, when the mouse 
     * pointer has exited the operable part of the drop site
     */
    public void dragExit( DropTargetEvent e ) 
    {
	    _dropComponent.undoDragUnderFeedback();
    }

    /**
     * called when ready to drop; undertake the transfer of data
     */
    public void drop( DropTargetDropEvent e ) 
    {
    	Transferable transferable = e.getTransferable();
	    DataFlavor chosen = chooseDropFlavor( e );
	    drop( e, chosen, transferable );
    }

    /**
     * undertake the transfer of data
     */
    public void drop( DropTargetDropEvent e, 
		              DataFlavor chosen, 
		              Transferable transferable ) 
    {
	    if( null == chosen ) 
	    {
            // no flavor match
	        e.rejectDrop();      	
    	    return;
	    }

	    // the actual operation
	    int dropAction = e.getDropAction();

	    // the actions that the source has specified with DragGestureRecognizer
	    int sourceAction = e.getSourceActions();      

	    if ( ( sourceAction & _acceptableDropAction ) == 0 ) 
	    {
	        // no action match found
    	    e.rejectDrop(); 
	        _dropComponent.undoDragUnderFeedback();
    	    return;
	    }
    
    	Object data;
	    try 
	    {
	        // the source listener receives this action in dragDropEnd.
    	    // if the action is DnDConstants.ACTION_COPY_OR_MOVE then
	        // the source receives MOVE!
    	    e.acceptDrop( e.getDropAction() );

    	    data = transferable.getTransferData( chosen );
	        if( null == data ) 
	        {
		        throw new NullPointerException();
	        }
	    }
	    catch ( Throwable t ) 
	    {
	        t.printStackTrace();
	        e.dropComplete(false);
	        _dropComponent.undoDragUnderFeedback();
            return;
	    }

        // insert into drop component
	    boolean status = _dropComponent.add( data, e, chosen );
	    if( status )
	    {
    	    e.dropComplete( true );      
    	}
    	else
    	{
    	    e.dropComplete( false );
    	}
	    _dropComponent.undoDragUnderFeedback();
    }

    /**
     * Set acceptable drop action.
     */
    public void setAcceptableDropAction( int action ) 
    {
	    _acceptableDropAction = action;
    }

    /**
     * Set acceptable drop flavors.
     */
    public void setAcceptableDropFlavors( DataFlavor[] flavors ) 
    {
    	_acceptableDropFlavors = flavors;
    }
    
    /**
     * Called by isDragOk
     * Checks to see if the flavor drag flavor is acceptable
     * @param e the DropTargetDragEvent object
     * @return whether the flavor is acceptable
     */
    private boolean isDragFlavorSupported( DropTargetDragEvent e ) 
    {
	    for( int i=0; i < _acceptableDropFlavors.length; i++ ) 
	    {
	        if( e.isDataFlavorSupported( _acceptableDropFlavors[i] ) )
		        return true;
	    }
	    return false;
    }

    /**
     * Choose the drop flavor from the event.
     */
    private DataFlavor chooseDropFlavor( DropTargetDropEvent e ) 
    {
	    DataFlavor[] flavors = e.getCurrentDataFlavors();
    	DataFlavor chosen = null;
	    for( int i=0; i < _acceptableDropFlavors.length; i++ ) 
	    {
	        if( e.isDataFlavorSupported( _acceptableDropFlavors[i] ) ) 
	        {
		        chosen = _acceptableDropFlavors[i];
		        break;
        	}
        }
	    return chosen;
    }

    /**
     * Called by dragEnter and dragOver
     * Checks the flavors and operations
     * @param e the event object
     * @return whether the flavor and operation is ok
     */
    private boolean isDragOk( DropTargetDragEvent e ) 
    {
	    if( !_dropComponent.isDragOk( e ) )
	        return false;

	    if( !isDragFlavorSupported( e ) ) 
	        return false;
      
	    // examine the drop action
	    int dropAction = e.getDropAction();      

	    // if drop action not valid  
	    if( (dropAction & _acceptableDropAction ) == 0 )
	        return false;
	        
	    return true;
    }

    /**
     * Reject the drop.
     */
    private void rejectDrag( DropTargetDragEvent e )
    {
	    _dropComponent.dragUnderFeedback( false, e );
	    e.rejectDrag();      
    }

}
