/**
 * Copyright 2001-2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */

package com.rational.rpw.environment;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
import java.util.Map;
import java.util.MissingResourceException;

import javax.swing.JFrame;

import com.rational.rpw.rpwapplication.RPWAlertDlg;
import com.rational.rpw.utilities.CommonFunctions;

/**
 * 
 * This class will store configuration settings.  Although this class will use the basic
 * functionality of ResourceBundles, it will only recognized UTF8 file encodings and it must be
 * located in the installation directory.  See RPWConfiguration for an concrete example....
 * 
 */
public abstract class ConfigurationFile
{
	public static final String KEY_VALUE_SEPARATOR = "=";
	public static final String COMMENT_VALUE = "#";
	
	private static final int STORAGE_SIZE = 300;

	/**
	 * If you use this contructor, you must call initializePropertiesFile manually
	 */	
	public ConfigurationFile()
	{
		//do nothing
	}
	
	/**
	 * Constructor
	 * @param pContext the current application context
	 * @param pFileFileName the property file name(excluding the installation directory)
	 * @param pResourceTable the Map to store the configuration file entries
	 */
	public ConfigurationFile(String pFileName, Map pResourceTable)
	{
		initializePropertiesFile(pFileName, pResourceTable, null);
	}
	
	/**
	 * Constructor
	 * @param pContext the current application context
	 * @param pFileFileName the property file name(excluding the installation directory)
	 * @param pResourceTable the Map to store the configuration file entries
	 * @param pKeyList the List to store the keys in the order they are read
	 */
	public ConfigurationFile(String pFileName, Map pResourceTable, List pKeyList)
	{
		initializePropertiesFile(pFileName, pResourceTable, pKeyList);
	}
	
	/**
	 * Reads in the property file from the installation directory
	 */
	protected void initializePropertiesFile(String pFileName, Map pResourceTable, List pKeyList)
	{
		// Get the Property Resource Bundle.
		try {
			// Get installation directory
			String theConfigFile = ""; //$NON-NLS-1$
			try {
				RegistryService theService = RegistryService.getInstance();
				theConfigFile = theService.getInstallDirectory() + File.separator + pFileName;
			} catch (EnvironmentException ee) {
				theConfigFile = pFileName;
				//RPWAlertDlg.showErrorMessage(Translations.getString("ENVIRONMENT_CONFIGURATIONFILE_3"),
				//							 Translations.getString("ENVIRONMENT_CONFIGURATIONFILE_4")); //$NON-NLS-1$ //$NON-NLS-2$
			}
			
			theConfigFile = CommonFunctions.formatPath(theConfigFile);
			readFile(theConfigFile, pResourceTable, pKeyList);
		} catch ( MissingResourceException mre ) {
			
		}
	}
	
	public abstract String getResourceString(String pKey);	
	
	//--------------------------------------------------------------------------------------------------
	// Read the property file assuming the specific encoding and return the parsed elements.
	// Comments in this file will begin with the "#"
	protected static void readFile(String propFile, Map pResourceTable, List pKeyList)
	{
		InputStreamReader isr;
		BufferedReader bis;
		FileInputStream fis;
		
		File theInputFile = new File(propFile);
		if (!theInputFile.exists()) {
			return;
		}

		try {
			fis = new FileInputStream(propFile);
			isr = new InputStreamReader(fis, StringConstants.fileEncoding);
			bis = new BufferedReader(isr);

			String line;
			String key, value;
			int pos;

			while ((line = bis.readLine()) != null)
			{
				// Skip empty lines and comments.
				if ( line.startsWith(COMMENT_VALUE) || line.length() == 0 ) //$NON-NLS-1$
				{
					continue;
				}
				else
				{
					// Break the line into it's component parts. Before the = is the key and everything
					// after is the value.
					if ( ( pos = line.indexOf(KEY_VALUE_SEPARATOR) ) != -1 ) //$NON-NLS-1$
					{
						// All comparisons will be done on the uppercase keys.
						key = line.substring(0, pos);
						//key = key.toUpperCase();
						value = line.substring(pos + 1);
						pResourceTable.put(key, value);
						if (pKeyList != null) {
							pKeyList.add(key);
						}
					}
					else
					{
						// Line is not in a recognizable format so just ignore it.
						continue;
					}
				}
			}
			
			fis.close();
			isr.close();
			bis.close();
			
		} catch ( IOException ioe ) {
			RPWAlertDlg.showErrorMessage(Translations.getString("ENVIRONMENT_CONFIGURATIONFILE_7"), Translations.getString("ENVIRONMENT_CONFIGURATIONFILE_8") + propFile + "=>" + ioe.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			return;
		}
	}
	
}
