package com.rational.rpw.environment;

import java.io.File;

/**
 * @author pdenno
 *
 * Copyright 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */

public class WindowsRegistry implements IRegistryService
{
	/**
	 * Load the library with the neccessary methods.
	 */
	private static boolean loaded; 
	
	static {
		try {
			System.loadLibrary("environment");
			loaded = true;
		} catch (Throwable te) {
			te.printStackTrace();
			// Lets sub-classes know that the loading of the native library was not successful.
			loaded = false;
		}
	}
	
	/**
	 * Native method declarations for the functions which are defined in the included
	 * library for getting the values of interest. In order for the native calls to work
	 * the fully resolved path name must be used. That is the name of the method in the 
	 * JNI DLL is com_rational_rpw_environment_WindowsRegistry_getRegistryValue.
	 * 
	 * Therefore you must always reference those methods from a class called
	 * com.rational.rpw.environment.WindowsRegistry.
	 */
	protected native String getRegistryValue(int topLevelKey, String key, String field);
	protected native String getWindowsTempDirectory();
	protected native int writeRegistryKey(int topLevelKey, String key, String field, String value);
	protected native int getDeviceCapsX();
	protected native int getDeviceCapsY();
		
	/**
	 * Top Level Registry Node definitions.
	 */
	public static final int HKEY_CLASSES_ROOT			= 1;
	public static final int HKEY_CURRENT_USER			= 2;
	public static final int HKEY_LOCAL_MACHINE			= 3;
	public static final int HKEY_USERS					= 4;
	public static final int HKEY_CURRENT_CONFIG			= 5;

	private String loadLibraryFailureErrorMessage 			= "Shared library for interfacing with OS is not loaded";
	
	/**
	 * Cache of the values from the registry. This is neccessary because
	 * the entire program crashes when the JNI is invoked multiple times.
	 * The conditions under which this occurs is that only certain keys
	 * cause the problem. For example if we ask the JNI to get the editor
	 * and viewer bindings for different file types then it will crash,
	 * but if we ask for the Builder Install directory multiple times 
	 * everything goes OK?? As a workaround I cache all the values so 
	 * that the JNI is only invoked once per key.
	 */
	private String processLibraryRoot						= null;
	private String repositoryDirectory						= null;
	private String htmlEditor								= null;
	private String gifEditor								= null;
	private String jpegEditor								= null;
	private String txtEditor 								= null;
	private String docEditor 								= null;
	private String htmlViewer								= null;
	private String gifViewer								= null;
	private String jpegViewer								= null;
	private String txtViewer								= null;
	private String docViewer								= null;
	private String builderInstallDirectory					= null;
	private String modellerInstallDirectory 				= null;
	private String organizerInstallDirectory				= null;
	private String rupLocation								= null;
	private String rcjdLocation							= null;
	private String rcndLocation 							= null;
	private String tempDirectory							= null;
			
	/**
	 * Constructor.
	 */
	public WindowsRegistry()
	{
		return;	
	}
	
	/**
	 * Get the value from the registry.
	 */
	private String getResult(int topLevelKey, String theKey, String theField)
		throws EnvironmentException
	{
		String theValue = null;

		try {
			theValue = this.getRegistryValue(topLevelKey, theKey, theField);
		}
		catch ( Throwable e ) {
			e.printStackTrace();
			throw new EnvironmentException("Error getting values: " + e.getMessage());
		}

		return stripInputArguments(theValue);
	}
	
	/**
	 * Windows uses the %1, etc.. notation to indicate argument substitution when calling 
	 * applications (i.e. iexplorer.exe %1). This however causes problems when we return 
	 * this information and try to invoke a browser because the browser also uses the % 
	 * sign as a substitution mechanism. Therefore this routine will strip these values 
	 * off before returning.
	 * 
	 * Another problem also occurs because windows puts quotes around names with spaces in
	 * them. 
	 */
	private String stripInputArguments(String input)
	{
		int index = input.indexOf("%");
		if ( index != -1 )
		{
			input = input.substring(0, index);		
		}
		
		// Trim any leading and trailing whitespace.
		input = input.trim();
		
		// Remove quotes at the beginning of the string.
		if ( input.startsWith("\"") )
		{
			input = input.substring(1, input.length());	
		}
		
		// Remove quotes at the end of the string also.
		if ( input.endsWith("\"") )
		{
			input = input.substring(0, input.length() - 1);
		}
					
		return input;
	}
	
	/**
	 * Retrieve the location of the default repository.
	 */
	public String getRepositoryDirectory()
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		if ( repositoryDirectory == null )
		{
			String RUPBUILDER_INSTALL_KEY		= "SOFTWARE\\Rational Software\\RUPBuilder";
			String RUPBUILDER_REPOSITORY_FIELD	= "Repository";
			repositoryDirectory = getResult(HKEY_LOCAL_MACHINE, 
						  					RUPBUILDER_INSTALL_KEY,
						  					RUPBUILDER_REPOSITORY_FIELD);	
			
			if ( !repositoryDirectory.endsWith(File.separator) )
			{
				repositoryDirectory += File.separator;	
			}
		}
		return repositoryDirectory;	
	}
	
	/**
	 * Retrieve the location of the process library root.
	 */
	public String getProcessLibrary()
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		if ( processLibraryRoot == null )
		{
			String STANDALONE_ORGANIZER_INSTALL_KEY	= "SOFTWARE\\Rational Software\\RUPOrganizer";
			String STANDALONE_PROCESS_LIBRARY_FIELD	= "ProcessLibrary";
			processLibraryRoot = getResult(HKEY_LOCAL_MACHINE,
						  				   STANDALONE_ORGANIZER_INSTALL_KEY,
						  				   STANDALONE_PROCESS_LIBRARY_FIELD);	
			
			if ( !processLibraryRoot.endsWith(File.separator) )
			{
				processLibraryRoot += File.separator;	
			}
		}
		return processLibraryRoot;
	}
	
	/**
	 * Get the application which is registered to handle files of the input type.
	 */
	public String getViewer(String fileEnding)
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		fileEnding = fileEnding.toUpperCase();
		
		if ( fileEnding.equals(".HTML")
			|| fileEnding.equals(".HTM") )
		{
			if ( htmlViewer == null )
			{
				String HTML_VIEWER_KEY	 = "htmlfile\\shell\\opennew\\command";
				String HTML_VIEWER_FIELD = "";
				htmlViewer = getResult(HKEY_CLASSES_ROOT,
									   HTML_VIEWER_KEY,
									   HTML_VIEWER_FIELD);
			}
			return htmlViewer;
		}
		else if ( fileEnding.equals(".GIF") )
		{
			if ( gifViewer == null )
			{
				String GIF_VIEWER_KEY	 = "giffile\\shell\\open\\command";
				String GIF_VIEWER_FIELD	 = "";
				gifViewer = getResult(HKEY_CLASSES_ROOT,
									  GIF_VIEWER_KEY,
									  GIF_VIEWER_FIELD);
			}
			return gifViewer;
		}
		else if ( fileEnding.equals(".JPEG") 
				  || fileEnding.equals(".JPG") )
		{
			if ( jpegViewer == null )
			{
				String JPEG_VIEWER_KEY	 = "jpegfile\\shell\\open\\command";
				String JPEG_VIEWER_FIELD = "";
				jpegViewer = getResult(HKEY_CLASSES_ROOT,
									   JPEG_VIEWER_KEY,
									   JPEG_VIEWER_FIELD);
			}
			return jpegViewer;
		}
		else if ( fileEnding.equals(".TXT") )
		{
			if ( txtViewer == null )
			{
				String TEXT_FILE_KEY	= "txtfile\\shell\\open\\command";
				String TEXT_FILE_FIELD	= "";
				txtViewer = getResult(HKEY_CLASSES_ROOT,
									  TEXT_FILE_KEY,
									  TEXT_FILE_FIELD);	
			}
			return txtViewer;
		}
		else if ( fileEnding.equals(".DOC") )
		{
			if ( docViewer == null )
			{
				String DOC_VIEWER_KEY 	= "WordDocument\\protocol\\StdFileEditing\\server";
				String DOC_VIEWER_FIELD = "";
				docViewer = getResult(HKEY_CLASSES_ROOT,
									  DOC_VIEWER_KEY,
									  DOC_VIEWER_FIELD);
			}
			return docViewer;	
		}
		else
		{
			return "";
		}
	}
	
	/**
	 * Get the application which is registered to handle files of the input type.
	 */
	public String getEditor(String fileEnding)
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		fileEnding = fileEnding.toUpperCase();
		
		if ( fileEnding.equals(".HTML") 
			|| fileEnding.equals(".HTM") )
		{
			if ( htmlEditor == null )
			{
				String HTML_EDITOR_KEY	 = "htmlfile\\shell\\Edit\\command";
				String HTML_EDITOR_FIELD = "";
				htmlEditor = getResult(HKEY_CLASSES_ROOT,
								  	   HTML_EDITOR_KEY,
								  	   HTML_EDITOR_FIELD);
			}
			return htmlEditor;
		}
		else if ( fileEnding.equals(".GIF") )
		{
			if ( gifEditor == null )
			{
				String GIF_EDITOR_KEY	 = "giffile\\shell\\open\\command";
				String GIF_EDITOR_FIELD	 = "";
				gifEditor = getResult(HKEY_CLASSES_ROOT,
									  GIF_EDITOR_KEY,
									  GIF_EDITOR_FIELD);
			}
			return gifEditor;	
		}
		else if ( fileEnding.equals(".JPEG") 
				  || fileEnding.equals(".JPG") )
		{
			if ( jpegEditor == null )
			{
				String JPEG_EDITOR_KEY	 = "jpegfile\\shell\\open\\command";
				String JPEG_EDITOR_FIELD = "";
				jpegEditor = getResult(HKEY_CLASSES_ROOT,
									   JPEG_EDITOR_KEY,
									   JPEG_EDITOR_FIELD);
			}
			return jpegEditor;	
		}
		else if ( fileEnding.equals(".TXT") )
		{
			if ( txtEditor == null )
			{
				String TEXT_FILE_KEY	= "txtfile\\shell\\open\\command";
				String TEXT_FILE_FIELD	= "";
				txtEditor = getResult(HKEY_CLASSES_ROOT,
									  TEXT_FILE_KEY,
									  TEXT_FILE_FIELD);
			}
			return txtEditor;	
		}
		else if ( fileEnding.equals(".DOC") )
		{
			if ( docEditor == null )
			{
				String DOC_EDITOR_KEY 	= "WordDocument\\protocol\\StdFileEditing\\server";
				String DOC_EDITOR_FIELD = "";
				docEditor = getResult(HKEY_CLASSES_ROOT,
									  DOC_EDITOR_KEY,
									  DOC_EDITOR_FIELD);
			}
			return docEditor;	
		}
		else
		{
			return "";	
		}
	}
	
	/**
	 * Get the directory where we store application specific information for builder.
	 */
	public String getBuilderApplicationDirectory()
		throws EnvironmentException
	{
		String applicationDirectory = System.getProperty("user.home")
							   + File.separator
							   + "Application Data"
							   + File.separator;
		
		// Check that this directory exists.
		File theDir = new File(applicationDirectory);
		if ( !theDir.exists() )
		{
			theDir.mkdirs();
		}
		
		return applicationDirectory + StringConstants.BUILDER_USER_STORAGE + File.separator;
	}
	
	/**
	 * Get the directory where we store application specific information for Modeller.
	 */
	public String getModellerApplicationDirectory()
		throws EnvironmentException
	{
		String applicationDirectory = System.getProperty("user.home")
							   + File.separator
							   + "Application Data"
							   + File.separator;
		
		// Check that this directory exists.
		File theDir = new File(applicationDirectory);
		if ( !theDir.exists() )
		{
			theDir.mkdirs();
		}
		
		return applicationDirectory + StringConstants.MODELER_USER_STORAGE + File.separator;	
	}	
	
	/**
	 * Get the directory where we store application specific information for the 
	 * standalone Organizer.
	 */
	public String getStandaloneOrganizerApplicationDirectory()
		throws EnvironmentException
	{
		String applicationDirectory = System.getProperty("user.home")
							   + File.separator
							   + "Application Data"
							   + File.separator;
		
		// Check that this directory exists.
		File theDir = new File(applicationDirectory);
		if ( !theDir.exists() )
		{
			theDir.mkdirs();
		}
		
		return applicationDirectory + StringConstants.ORGANIZER_USER_STORAGE + File.separator;	
	}
	
	/**
	 * Get the directory where we store application specific information for the 
	 * extended help application. This method is never called when extended help
	 * is running.
	 */
	public String getExtendedHelpApplicationDirectory()	
		throws EnvironmentException
	{
		return "";	
	}
	
	/**
	 * Get the install location for Builder.
	 */
	public String getBuilderInstallDirectory()
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		if ( builderInstallDirectory == null )
		{
			String RUPBUILDER_INSTALL_FIELD	= "Install";
			String RUPBUILDER_INSTALL_KEY	= "SOFTWARE\\Rational Software\\RUPBuilder";
			builderInstallDirectory = getResult(HKEY_LOCAL_MACHINE,
							 					RUPBUILDER_INSTALL_KEY,
							 					RUPBUILDER_INSTALL_FIELD);	
			
			if ( !builderInstallDirectory.endsWith(File.separator) )
			{
				builderInstallDirectory += File.separator;	
			}
		}
		return builderInstallDirectory;
	}
	
	/**
	 * Get the install location for Modeller.
	 */
	public String getModellerInstallDirectory()
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		if ( modellerInstallDirectory == null )
		{
			String MODELLER_INSTALL_KEY			= "SOFTWARE\\Rational Software\\XDE\\AddIns\\RUP Modeler";
			String MODELLER_INSTALL_FIELD		= "InstallDir";
			modellerInstallDirectory = getResult(HKEY_LOCAL_MACHINE,
							 					 MODELLER_INSTALL_KEY,
							 					 MODELLER_INSTALL_FIELD);
			
			if ( !modellerInstallDirectory.endsWith(File.separator) )
			{
				modellerInstallDirectory += File.separator;		
			}
		}
		return modellerInstallDirectory;
	}
	
	/**
	 * Get the install location for standalone organizer.
	 */
	public String getStandaloneOrganizerInstallDirectory()
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		if ( organizerInstallDirectory == null )
		{
			String STANDALONE_ORGANIZER_INSTALL_KEY		= "SOFTWARE\\Rational Software\\RUPOrganizer";
			String STANDALONE_ORGANIZER_INSTALL_FIELD	= "Install";
			organizerInstallDirectory = getResult(HKEY_LOCAL_MACHINE,
							 					  STANDALONE_ORGANIZER_INSTALL_KEY,
							 					  STANDALONE_ORGANIZER_INSTALL_FIELD); 	
			
			if ( !organizerInstallDirectory.endsWith(File.separator) )
			{
				organizerInstallDirectory += File.separator;
			}  
		}
		return organizerInstallDirectory;
	}	
	
	/**
	 * Get the install location for the extended help application.
	 * Currently this information is never required by the extended help
	 * application.
	 */
	public String getExtendedHelpInstallDirectory()
		throws EnvironmentException
	{
		return "";	
	}
	
	
	/**
	 * Get the location of the temp directory as read from the registry.
	 */
	public String getTempDirectory()
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		/**
		 * Get the temp directory from the local system.
		 */
		if ( tempDirectory == null )
		{
			tempDirectory = getWindowsTempDirectory();
				    	
		    /**
		 	 * This is a shortcut that I have added but is not widely used and should 
		 	 * only be used when all else has failed.
		 	 */
			if ( tempDirectory == null )
		    {
				String TEMP_DIRECTORY_KEY	= "SOFTWARE\\Rational Software\\RUPBuilder";
				String TEMP_DIRECTORY_FIELD	= "tempDirectory";
				tempDirectory = getResult(HKEY_LOCAL_MACHINE,
										  TEMP_DIRECTORY_KEY,
								  		  TEMP_DIRECTORY_FIELD);	
		    }
		}
		return tempDirectory;
	}
	
	/**
	 * Gets the location where the RUP is installed on the client machine.
	 */
	public String getRUPInstallDirectory()
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		/**
		 * Look for Japanese first.
		 */
		try {
			if ( rupLocation == null )
			{
				String RUP_JAP_INSTALL_FIELD = "RootDir";
				String RUP_JAP_INSTALL_KEY	= "SOFTWARE\\Rational Software\\Rational Unified Process.ja-JP";
				rupLocation = getResult(HKEY_LOCAL_MACHINE,
								  		RUP_JAP_INSTALL_KEY,
								  		RUP_JAP_INSTALL_FIELD);
				return rupLocation;
			}
		} catch ( EnvironmentException ee ) {
			// Just ignore, we will try again to find the english location.	
		}					  
		
		/**
		 * Japanese not found then default to the english.
		 */
		if ( rupLocation == null )
		{
			String RUP_INSTALL_FIELD = "RootDir";
			String RUP_INSTALL_KEY	 = "SOFTWARE\\Rational Software\\Rational Unified Process\\5.1";
			rupLocation =  getResult(HKEY_LOCAL_MACHINE,
								     RUP_INSTALL_KEY,
							  		 RUP_INSTALL_FIELD);	
		}
		return rupLocation;
	}
	
	/**
	 * Get the location of where the rjcd is located.
	 */
	public String getRUPEclipseInstallDirectory()
		throws EnvironmentException	
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		if ( rcjdLocation == null )
		{
			String RUP_ECLIPSE_INSTALL_KEY  = "SOFTWARE\\Rational Software\\Rational Unified Process.rcjd";
			String RUP_INSTALL_FIELD		= "RootDir";
			rcjdLocation = getResult(HKEY_LOCAL_MACHINE,
							 	 	 RUP_ECLIPSE_INSTALL_KEY,
							  		 RUP_INSTALL_FIELD);	
		}
		return rcjdLocation;
	}
	
	/**
	 * Get the location of where the rcnd is located.
	 */
	public String getRUPDotNetInstallDirectory()
		throws EnvironmentException
	{
		// Check that the shared library is loaded.
		if ( !this.loaded )
		{
			throw new EnvironmentException(loadLibraryFailureErrorMessage);
		}
		
		if ( rcndLocation == null )
		{
			String RUP_DOTNET_INSTALL_KEY	= "SOFTWARE\\Rational Software\\Rational Unified Process.rcnd";
			String RUP_INSTALL_FIELD		= "RootDir";
			rcndLocation = getResult(HKEY_LOCAL_MACHINE,
							  		 RUP_DOTNET_INSTALL_KEY,
							  		 RUP_INSTALL_FIELD);	
		}
		return rcndLocation;
	}	
}
