package com.rational.rpw.processtools.contextrupmapper;

import java.io.File;
import java.net.URL;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.rational.rpw.processview.Context;
import com.rational.rpw.processviewer.ViewerUtilities;

/**
 * This class searches for available context term to rup term xml files,
 * and maps the given context appropriately to a string of words that
 * are applicable for extended help to search with.<br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class ContextRupMapper extends DefaultHandler
{
	private static final String XML_SUFFIX = ".xml";
	private static final String TOOL_ELEMENT = "tool";
	private static final String TOPIC_ELEMENT = "topic_term";
	private static final String TOPIC_REPLACEMENT_ELEMENT = "replacement_term";
	private static final String TOOL_REPLACEMENT_ELEMENT = "tool_replacement_term";	
	
	// string buffer of words
	private StringBuffer _wordBuffer = new StringBuffer();
	
	// tool replacements
	private Vector _toolReplacements = new Vector();

	// topic term replacements
	private Vector _topicReplacements = new Vector();

	// original topics
	private Vector _originalTopics = new Vector();
	
	// original tool
	private String _originalTool = null;
	
	// running element name
	private String _elementName = null;
	
	// parse tool block
	private boolean _isParseToolBlock = false;
	
	// parse topic block
	private boolean _isParseTopicBlock = false;
	
	// xml directory
	private String _xmlDirectory = null;
	
	// temporary original topic holder
	private String _topicToRemove = null;
	
			
	/**
	 * Default constructor.
	 */
	public ContextRupMapper( String xmlDirectory,
	                          Context context )
	{
		// initialize terms
		_originalTool = context.getTool();
		Iterator topics = context.getTopics();
		while( topics.hasNext() )
		{
			_originalTopics.addElement( topics.next() );
		}
		
		_xmlDirectory = xmlDirectory;
	}

	/**
	 * Parse and construct string.
	 */
	public String getMappedString()
	{		
		// parse files
		parseFiles();
		
		// construct final string        
        constructStringQuery();
        
        return( _wordBuffer.toString() );			
	}
		
    //===========================================================
    // SAX DocumentHandler methods
    //===========================================================
    public void startElement( String namespaceURI,
                               String lName, // local name
                               String qName, // qualified name
                               Attributes attrs )
    	throws SAXException
    {
        _elementName = qName.trim();
    }

    public void endElement( String namespaceURI,
                             String sName, // simple name
                             String qName  // qualified name
                          )
    	throws SAXException
    {
        _elementName = qName.trim();
     	if( _elementName.equals( TOOL_ELEMENT ) )
     	{
     		_isParseToolBlock = false;	
     	}
     	else if( _elementName.equals( TOPIC_ELEMENT ) )
     	{
     		_isParseTopicBlock = false;	
     	}
    }

    public void characters( char buf[], 
                             int offset, 
                             int len )
    	throws SAXException
    {
        String value = new String( buf, offset, len ).trim();
        if( value.length() != 0 )
        {
        	if( _elementName.equals( TOOL_ELEMENT ) )
        	{
        		if( value.equals( _originalTool ) )
        		{
        			_isParseToolBlock = true;        			
        		}
        	}
        	else if( _elementName.equals( TOPIC_ELEMENT ) )
        	{
        		//if( _originalTopics.contains( value ) )
        		// first tokenized value by spaces
        		StringTokenizer valueTokenizer = new StringTokenizer( value );
        		Vector valueTokens = new Vector();
        		while( valueTokenizer.hasMoreTokens() )
        		{
        			valueTokens.addElement( valueTokenizer.nextToken() );
        		}
        		// now tokenize topics
       			Vector tokenizedTopics = new Vector();
        		for( int i = 0; i < _originalTopics.size(); i++ )
        		{
        			String topic = (String)_originalTopics.elementAt( i );
        			//System.out.println( "original topic = " + topic );
        			StringTokenizer tokenizer = new StringTokenizer( topic );
        			while( tokenizer.hasMoreTokens() )
        			{
        				tokenizedTopics.addElement( tokenizer.nextToken() );
        			}        			
        		}
        		
        		// now do the comparison test
        		if( tokenizedTopics.containsAll( valueTokens ) )        		
        		{
        			_isParseTopicBlock = true;	
        			// remove the element name from the original topics
        			_topicToRemove = value;
        		}        			
        	}
        	else if( _elementName.equals( TOPIC_REPLACEMENT_ELEMENT ) )
        	{
        		if( _isParseToolBlock && _isParseTopicBlock )
        		{
        			//System.out.println( "adding to topic replacement list = " + value );
        			_topicReplacements.addElement( value );
        			if( null != _topicToRemove )
        			{
        				//System.out.println( "removing from original topic = " + _topicToRemove );
        				_originalTopics.remove( _topicToRemove );
        			}
        			
        		}
        	}
        	else if( _elementName.equals( TOOL_REPLACEMENT_ELEMENT ) )
        	{
        		if( _isParseToolBlock )
        		{
        			_toolReplacements.addElement( value );
        		}
        	}
        }
    }

	/**
	 * Parses all xml files in the given directory.
	 */
	private void parseFiles()
	{
		Vector xmlFileList = ViewerUtilities.getFileList( _xmlDirectory, XML_SUFFIX );
		for( int i = 0; i < xmlFileList.size(); i++ )
		{			
			String fileName = (String)xmlFileList.elementAt( i );
	        // sse the default (non-validating) parser
	        SAXParserFactory factory = SAXParserFactory.newInstance();
	        try 
	        {
	            // Parse the input
	            SAXParser saxParser = factory.newSAXParser();
	            if( _xmlDirectory.startsWith( "http" ) )
	            {	            	
					URL fullUrl = new URL( new URL( _xmlDirectory ), fileName );
	            	saxParser.parse( fullUrl.openStream(), this );
	            }
	            else
	            {
					String fullName = ViewerUtilities.convertFileSpacing(
										ViewerUtilities.convertFilename( _xmlDirectory ) )+
										File.separator +
									  	fileName;
					if( fullName.startsWith( "\\\\" ) )
					{
						// handle UNC
						//fullName = fullName.replaceAll( "\\", "/" );
						URL fullUrl = new URL( "file", "", fullName );
						saxParser.parse( fullUrl.openStream(), this );
					}
					else
					{
	            		saxParser.parse( fullName, this );
					}
	            }
	        } 
	        catch (Throwable t) 
	        {
	            t.printStackTrace();
	            // do nothing here; eventually, original query is simply used
	        }
		}
		
	}
	
	/**
	 * Constructs the final string.
	 */
	private void constructStringQuery()
	{
        // first see if there's any replacement for the tool
        if( _toolReplacements.size() == 0 )
        {
        	// none, so just use original tool
        	_wordBuffer.append( "\"" + _originalTool + "\"" );
       		_wordBuffer.append( " " );
        }
        else
        {
	        for( int i = 0; i < _toolReplacements.size(); i++ )
    	    {
        		_wordBuffer.append( "\"" + _toolReplacements.elementAt( i ) + "\"" );
        		_wordBuffer.append( " " );
	        }
        }
        
        // now see if there's any replacement for the topic
        for( int i = 0; i < _topicReplacements.size(); i++ )
	    {
    		_wordBuffer.append( "\"" + _topicReplacements.elementAt( i ) + "\"" );
     		_wordBuffer.append( " " );
        }

    	// now use remainder of original topics
    	//tokenize on space for wider searching
		Vector tokenizedTopics = new Vector();
		for( int i = 0; i < _originalTopics.size(); i++ )
		{
			String topic = (String)_originalTopics.elementAt( i );
			StringTokenizer tokenizer = new StringTokenizer( topic );
			while( tokenizer.hasMoreTokens() )
			{
				tokenizedTopics.addElement( tokenizer.nextToken() );
			}        			
		}
    	
    	// none, so just use original topics
    	for( int i = 0; i < tokenizedTopics.size(); i++ )
    	{
        	_wordBuffer.append( "\"" + tokenizedTopics.elementAt( i ) + "\"" );    		
    		_wordBuffer.append( " " );
    	}
	}
	
	/**
	 * For testing
	 */
    public static void main(String argv[])
    {
        if (argv.length != 1) 
        {
            System.err.println("Usage: cmd filename");
            System.exit(1);
        }
        
        // Use an instance of ourselves as the SAX event handler
        //DefaultHandler handler = new ContextRupMapper();
        Context context = new Context();
        context.addTopic( "Any" );
        context.setTool( "ReqPro" );
        ContextRupMapper mapper = new ContextRupMapper( "e:\\temp\\published\\ext_help\\", context );
		System.out.println( mapper.getMappedString() );
                                                         
        System.exit(0);
    }


}
