package com.rational.rpw.processview;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.StringTokenizer;
import java.util.Vector;

import com.rational.rpw.utilities.CommonFunctions;


/**
 * This class provides methods to read (serialize) and write
 * (deserialize) bookmarks. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class BookmarkLibrary
{
	// default bookmark library for publishing and reading
	public static final String LIBRARY_DIRECTORY = "applet"; //$NON-NLS-1$
	
	public static final String BOOKMARK_SUFFIX = ".bkm"; //$NON-NLS-1$
	public static final String SPACE_REPLACEMENT = "_";	 //$NON-NLS-1$
	public static final String SPACE_STRING = " ";	 //$NON-NLS-1$

	public static final String DEFAULT_BOOKMARK_CFG_NAME = "DefaultBookmark.cfg"; //$NON-NLS-1$
	public static final String PUBLISHED_BOOKMARKS_CFG_NAME = "PublishedBookmarks.cfg"; //$NON-NLS-1$
	public static final String PUBLISHED_BOOKMARKS_DELIMITER = "*"; //$NON-NLS-1$
		
	/**
	 * Default constructor.
	 */
	private BookmarkLibrary()
	{
	}
	
	/**
	 * Determines whether the bookmark with the given name in
	 * the given directory, exists.
	 */
	public static boolean isBookmarkExist( String bookmarkName,
                                           String destinationDirectory )
    {
    	boolean isExist = false;
    	
    	String libraryDirectory = getLibraryDirectory( destinationDirectory );
	    
        // replace spaces with proper character
        String finalBookmarkName = bookmarkName.replace( SPACE_STRING.charAt( 0 ), SPACE_REPLACEMENT.charAt( 0 ) );
        
	    String fileName = libraryDirectory + 
	    				  finalBookmarkName +
	    				  BOOKMARK_SUFFIX;
                       
 		// does the enclosing directory exists
 		File directory = new File( libraryDirectory );			
		if (!directory.exists()) 
		{
			isExist = false;
		}
		else
		{
			// does the file exist
			File file = new File( fileName );
			if( file.exists() )
			{
				isExist = true;
			}
		}
		
		return( isExist );    	
    }
    
	/**
	 * Finds all the bookmarks in the given directory that
	 * matches a prefix. If the prefix is null, then all bookmarks
	 * that are found are returned.
	 * @param Vector	a list of found bookmarks
	 */
	public static Vector findBookmarks( String prefix,
	                                    String directory )
 			throws FileNotFoundException, IOException, ClassNotFoundException
	{
		if( null != prefix )
		{
			prefix = prefix.replace( BookmarkLibrary.SPACE_STRING.charAt( 0 ),
		 	                         BookmarkLibrary.SPACE_REPLACEMENT.charAt( 0 ) );
		}

		Vector bookmarkList = new Vector();
		// read in all bookmarks from the directory
		File libDirectory = new File( directory );
		if( libDirectory.exists() )
		{
			String[] fileNameList = libDirectory.list();
			
			// now parse and only take those that match the configuration
			for( int i = 0; i < fileNameList.length; i++ )
			{	
				if( null != prefix )
				{
					if( fileNameList[ i ].startsWith( prefix ) &&				
				        fileNameList[ i ].endsWith( BookmarkLibrary.BOOKMARK_SUFFIX ) )
				    {
						String filename = fileNameList[ i ];
						String bookmarkName = filename.substring( 0, 
			 	                                         		  filename.indexOf( BookmarkLibrary.BOOKMARK_SUFFIX ) );
						Bookmark existing = BookmarkLibrary.readBookmark( directory,
						 	                                              bookmarkName );					
						// set existing to default
						existing.setDefault( true );
						bookmarkList.addElement( existing );				    	
				    }
				}
				else if( fileNameList[ i ].endsWith( BookmarkLibrary.BOOKMARK_SUFFIX ) )
				{
					String filename = fileNameList[ i ];
					String bookmarkName = filename.substring( 0, 
		 	                                         		  filename.indexOf( BookmarkLibrary.BOOKMARK_SUFFIX ) );
					Bookmark existing = BookmarkLibrary.readBookmark( directory,
					 	                                              bookmarkName );					
					// set existing to default
					existing.setDefault( true );
					bookmarkList.addElement( existing );							
				}				
			}
		}
		
		return( bookmarkList );
	}
	
    /**
     * Serializes the given bookmark object into a file in the
     * destinations directory.
     */
    public static void writeBookmark( Bookmark bookmark,
                                      String bookmarkName,
                                      String destinationDirectory )
 			throws FileNotFoundException, IOException
    {
    	String libraryDirectory = getLibraryDirectory( destinationDirectory );
	    
        // replace spaces with proper character
        String finalBookmarkName = bookmarkName.replace( SPACE_STRING.charAt( 0 ), SPACE_REPLACEMENT.charAt( 0 ) );
        
	    String fileName = libraryDirectory + 
	    				  finalBookmarkName +
	    				  BOOKMARK_SUFFIX;
	    				  
 		// make sure the enclosing directory exists 		
 		File directory = new File( libraryDirectory );
			
		if (!directory.exists()) 
		{
			directory.mkdirs();
		}
		directory = null;
				
		File file = new File( fileName );
		if( file.exists() )
		{
			file.delete();
		}
		file = null;
		
        // opens a file output stream and serializes the file
        FileOutputStream fos = null;
    	ObjectOutputStream oos = null;
        fos = new FileOutputStream(fileName);
   	    oos = new ObjectOutputStream(fos);

   	    oos.writeObject( bookmark );

        fos.close();
        oos.close();
        
//-----------------------------------------------------------------------------
//		String xmlFileName = libraryDirectory + finalBookmarkName + ".xml";    
//        
//		File xmlFile = new File( xmlFileName );
//		if( xmlFile.exists() )
//		{
//			xmlFile.delete();
//		}
//		XMLDataSerializer.persistObject(
//			xmlFile,
//			bookmark,
//			XMLDataSerializer.getBookmarkPersistenceDelegate());
//			
//		xmlFile = null;
//-----------------------------------------------------------------------------
		
// TO DEBUG *******************************************************************
//		XMLEncoder e = new XMLEncoder(System.out);
//		e.setPersistenceDelegate(bookmark.getClass(),
//								 new BookmarkPersistenceDelegate());
//
//		e.writeObject(bookmark);
//		e.flush();
//		e.close();
// ****************************************************************************    
    }

	/**
	 * Reads from a persisted bookmark.
	 */
	public static Bookmark readBookmark( String inputDirectory,
	                                     String bookmarkName )
 			throws FileNotFoundException, IOException, ClassNotFoundException
	{
		String libraryDirectory = getLibraryDirectory( inputDirectory );

        Bookmark bookmark = null;
        
        // replace space replacement with space
        String finalBookmarkName = bookmarkName.replace( SPACE_STRING.charAt( 0 ), SPACE_REPLACEMENT.charAt( 0 ) );

        String filename = libraryDirectory +
                          finalBookmarkName +
                          BOOKMARK_SUFFIX;
                          
       	// Desearialize the layout and pass it to the other constructor.
        FileInputStream fis = new FileInputStream( filename );
       	ObjectInputStream ois = new ObjectInputStream( fis );

        // Attempt to read the data that has been serialized out to disk.
   	    bookmark = (Bookmark)ois.readObject();

       	// Close the input streams.
        fis.close();
   	    ois.close();	        
        
        return( bookmark );
	}
	
	/**
	 * Reads from a persisted bookmark.
	 */
	public static Bookmark readBookmark( URL documentBase,
	                                     String bookmarkName )
			throws MalformedURLException, IOException, ClassNotFoundException
	{
        Bookmark bookmark = null;
        
        // replace space replacement with space
        String finalBookmarkName = bookmarkName.replace( SPACE_STRING.charAt( 0 ), SPACE_REPLACEMENT.charAt( 0 ) );
                       
        String filename = finalBookmarkName + BOOKMARK_SUFFIX;
     		
     	// handle spaces in filename if in http
    	String urlFilename = null;
    	if( documentBase.getProtocol().equals( "http" ) ) //$NON-NLS-1$
    	{
			StringTokenizer tokenizer = new StringTokenizer( filename );
			StringBuffer tempName = new StringBuffer();
			while( tokenizer.hasMoreTokens() )
			{
				tempName.append( tokenizer.nextToken() );
				if( tokenizer.hasMoreTokens() )
				{
					tempName.append( "%20" ); //$NON-NLS-1$
				}
			}
			urlFilename = tempName.toString();
   		}
   		else
   		{
   			urlFilename = filename;
   		}

        InputStream inStream = new URL( documentBase, urlFilename ).openStream();
       	ObjectInputStream ois = new ObjectInputStream( inStream );

        // Attempt to read the data that has been serialized out to disk.
   	    bookmark = (Bookmark)ois.readObject();

       	// Close the input streams.
        inStream.close();
   	    ois.close();	        
        
        return( bookmark );
	}
	
	/**
	 * Deletes the specified bookmark.
	 */
	public static boolean deleteBookmark( String bookmarkName,
	                                      String directory )
	{
		try
		{
        	// replace spaces with proper character
        	String finalBookmarkName = bookmarkName.replace( SPACE_STRING.charAt( 0 ), SPACE_REPLACEMENT.charAt( 0 ) );
               
			String filename = directory +
			                  File.separator +
			                  finalBookmarkName +
			                  BOOKMARK_SUFFIX;
        
			File file = new File( filename );
			boolean status = file.delete();
			
			if( !status )
			{
				return( false );
			}
			
			file = null;
		}
		catch( Exception e )
		{
			e.printStackTrace();
			return( false );
		}
		return( true );
	}
	
    /**
     * Writes the bookmark name to the default bookmark config file.
     */
    public static void writePublishedBookmarkNames( Vector bookmarks,
                                                      String destinationDirectory )
 			throws FileNotFoundException, IOException
    {
    	String libraryDirectory = getLibraryDirectory( destinationDirectory );
	    
	    Vector names = new Vector();
	    for( int i = 0; i < bookmarks.size(); i++ )
	    {
	    	String bookmarkName = ((Bookmark)bookmarks.elementAt( i )).getPresentationName();
	        // replace spaces with proper character
    	    String finalBookmarkName = bookmarkName.replace( SPACE_STRING.charAt( 0 ), SPACE_REPLACEMENT.charAt( 0 ) );        
	    	String fileName = finalBookmarkName +
	    					  BOOKMARK_SUFFIX;
			names.add( fileName );
	    }
                       
 		// make sure the enclosing directory exists
 		File directory = new File( libraryDirectory );
			
		if (!directory.exists()) 
		{
			directory.mkdirs();
		}
		directory = null;
		 
		File file = new File( libraryDirectory + PUBLISHED_BOOKMARKS_CFG_NAME );
		if( file.exists() )
		{
			file.delete();
		}
		file = null;
		
        // opens a file output stream and write the names out		
        FileOutputStream fos = new FileOutputStream( libraryDirectory + PUBLISHED_BOOKMARKS_CFG_NAME,
		                                             false );
    	ObjectOutputStream oos = new ObjectOutputStream( fos );

   	    oos.writeObject( names );		            	

		fos.close();
		oos.close();
		
    }

	/**
	 * Reads the published bookmark file names.
	 */
	public static Vector readPublishedBookmarkNames( URL documentBase )
			throws MalformedURLException, IOException, ClassNotFoundException
	{
    	InputStream inStream = new URL( documentBase, PUBLISHED_BOOKMARKS_CFG_NAME ).openStream();
       	ObjectInputStream ois = new ObjectInputStream( inStream );

        // Attempt to read the data that has been serialized out to disk.
   	    Vector names = (Vector)ois.readObject();

       	// Close the input streams.
        inStream.close();
   	    ois.close();	 
   	    inStream = null;               		

    	return( names );
	}
	
    /**
     * Writes the bookmark name to the default bookmark config file.
     */
    public static void writeDefaultBookmarkName( String bookmarkName,
                                                   String destinationDirectory )
 			throws FileNotFoundException, IOException
    {
    	String libraryDirectory = getLibraryDirectory( destinationDirectory );
	    
        // replace spaces with proper character
        String finalBookmarkName = bookmarkName.replace( SPACE_STRING.charAt( 0 ), SPACE_REPLACEMENT.charAt( 0 ) );        
	    String fileName = finalBookmarkName +
	    				  BOOKMARK_SUFFIX;
                       
 		// make sure the enclosing directory exists
 		File directory = new File( libraryDirectory );
			
		if (!directory.exists()) 
		{
			directory.mkdirs();
		}
		directory = null;
		 
		File file = new File( libraryDirectory + DEFAULT_BOOKMARK_CFG_NAME );
		if( file.exists() )
		{
			file.delete();
		}
		file = null;
		
        // opens a file output stream and write the name out
        FileOutputStream fos = new FileOutputStream( libraryDirectory + DEFAULT_BOOKMARK_CFG_NAME,
		                                             false );
    	ObjectOutputStream oos = new ObjectOutputStream( fos );

   	    oos.writeObject( fileName );		            	

		fos.close();
		oos.close();
		
    }

	/**
	 * Reads the default bookmark file name.
	 */
	public static String readDefaultBookmarkName( URL documentBase )
			throws MalformedURLException, IOException, ClassNotFoundException
	{
        /*InputStream inStream = new URL( documentBase, DEFAULT_BOOKMARK_CFG_NAME ).openStream();
  		// note this works for over the network  	
		StringBuffer result = new StringBuffer();    
		int c;
		while( (c = inStream.read() ) != -1 )
		{
			result.append( (char)c );
		}
		String content = result.toString();
    	
    	// close stream
    	inStream.close();
    	inStream = null;*/
    	
	    InputStream inStream = new URL( documentBase, DEFAULT_BOOKMARK_CFG_NAME ).openStream();
       	ObjectInputStream ois = new ObjectInputStream( inStream );

        // Attempt to read the data that has been serialized out to disk.
   	    String content = (String)ois.readObject();

       	// Close the input streams.
        inStream.close();
   	    ois.close();	 
   	    inStream = null;               		
    	
    	return( content );
	}
	
	/**
	 * Converts bookmark file name to presentation name.
	 */
	public static String convertFileToPresentationName( String bookmarkFilename )
	{
		// replace space replacement with space
        String firstStep = bookmarkFilename.replace( SPACE_REPLACEMENT.charAt( 0 ), SPACE_STRING.charAt( 0 )  );
        // remove suffix
        String secondStep = firstStep.substring( 0, firstStep.indexOf( BOOKMARK_SUFFIX ) );
        return( secondStep );
	}
	
	/**
	 * Construct library path.
	 */
	private static String getLibraryDirectory( String libraryRoot )
	{
        String tempDir = CommonFunctions.formatPath( libraryRoot );

		String libraryDirectory = null;
        if( !tempDir.endsWith( File.separator ) )
        {
    	    libraryDirectory = tempDir + 
    	                       File.separator;
        }
        else
        {
            libraryDirectory = tempDir;
        }
        
        return( libraryDirectory );
	}
}

