package com.rational.rpw.processview;

import javax.swing.ImageIcon;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Set;
import java.util.HashMap;
import java.util.Iterator;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipEntry;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.File;

/**
 * This class manages icons for the process view applications. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class IconManager 
{
	// Default icon
	public ImageIcon theDefaultIcon;
	
	// The base directory of the html page containing this control.  
	// This must be a valid URL path.
	protected String theBaseDirectory;
	
	// The parent directory of the base that contains this control.
	// This is another path where images may be found.
	protected String theParentDirectory;
	
    // the resource file
    protected URL theResourceFile;
    
	/**
	 * Default constructor.  Does nothing.
	 */
	public IconManager() 
	{
		super();
	}

	/**
	 * Constructor
	 * @param pBaseDirectory the base directory of the html page 
	 *                       containing this control.  This must be a valid URL path.
	 */
	public IconManager(String pBaseDirectory) 
	{
		theBaseDirectory = pBaseDirectory;
		initializeDefaultIcons();			
	}

	/**
	 * Constructor
	 * @param pBaseDirectory the base directory of the html page 
	 *                       containing this control.  This must be a valid URL path.
	 * @param pParentDirectory the parent directory of the html page 
	 *                         containing this control.  This must be a valid URL path.
	 */
	public IconManager(String pBaseDirectory,
	                   String pParentDirectory) 
	{
		theBaseDirectory = pBaseDirectory;
		theParentDirectory = pParentDirectory;
		initializeDefaultIcons();			
	}

	/**
	 * Returns an ImageIcon
	 * @param pPath path to an icon. If the path doesn't exist relative
	 *              to the base directory, then it tries to get it by treating
	 *              pPath as an absolute path.
	 * @return ImageIcon if one exists, null otherwise
	 */
	public ImageIcon getIcon(String pPath) 
	{
		ImageIcon theIcon = null;
		try
		{
			// first try to use given path as full path
			theIcon = getIconFromFullPath( convertFilenameToUNC( pPath ) );
       		if( null == theIcon || theIcon.getIconWidth() == -1 )
       		{
       			// doesn't exist, now try it with base directory       			
				theIcon = 	getIconFromFullPath( convertFilenameToUNC( theBaseDirectory + pPath ) );
       			if( null == theIcon || theIcon.getIconWidth() == -1 )
       			{
       				// doesn't exist, now try it with parent directory  
					theIcon = 	getIconFromFullPath( convertFilenameToUNC( theParentDirectory + pPath ) );
       			}
       		}
			
		}
		catch( Exception e )
		{
		}
		return theIcon;
	}

	/**
	 * Returns an ImageIcon.  This is a static method accessible by any method.
	 * @param pBaseDirectory The base directory of the html file containing this applet. 
	 * @param pRelativePath path to an icon relative to the theBaseDirectory
	 * @return ImageIcon if one exists, default image otherwise
	 */
	public static ImageIcon getIcon(String pBaseDirectory, String pRelativePath)
	{
		ImageIcon theIcon = null;
		try 
		{
			URL theIconPath = new URL( pBaseDirectory + pRelativePath );
			theIcon = new ImageIcon( theIconPath );
		} 
		catch (MalformedURLException mue) 
		{
		}	

		return theIcon;

	}

	/**
	 * Returns an ImageIcon
	 * @param pPath path to an icon given the full path
	 * @return ImageIcon if one exists, null otherwise
	 */
	public ImageIcon getIconFromFullPath(String pFullPath) 
	{
		ImageIcon theIcon = null;
		// convert any UNC separators to the system default one
		String convertedName = pFullPath.replace( File.separator.charAt( 0 ), 
		                                          '/' );
		try 
		{
			URL theIconPath = null;
			if( convertedName.startsWith( "file://" ) )//$NON-NLS-1$
			{
				// make it so that the "file" is extracted, this is for UNC purposes
				theIconPath = new URL( "file", "", convertedName.substring( 5 ) );
			}
			else if( convertedName.startsWith( "http:" ) || //$NON-NLS-1$
					  convertedName.startsWith( "https:" ) || //$NON-NLS-1$
			          convertedName.startsWith( "file:" ) ) //$NON-NLS-1$
			{
				// this came from a web server or local applet somewhere
				// replace spaces by standard %20
				if( convertedName.indexOf( " " ) > 0 )
				{
					String name = convertedName.replaceAll( " ", "%20" );
					theIconPath = new URL( name );
				}
				else
				{
					theIconPath = new URL( convertedName );
				}
			}
			else
			{
				// this came from a file somewhere
				theIconPath = new URL( "file", "", convertedName ); //$NON-NLS-1$ //$NON-NLS-2$
			}
			theIcon = new ImageIcon( theIconPath );
		} 
		catch (MalformedURLException mue) 
		{
		}		
		catch( Exception e )
		{
		}

		return theIcon;
	}


	/**
	 * Initializes the default icons
	 */
	protected void initializeDefaultIcons() 
	{
		try 
		{
			URL theIcon = new URL( theBaseDirectory + "default.gif" ); //$NON-NLS-1$
			theDefaultIcon = new ImageIcon( theIcon );
		} 
		catch (MalformedURLException mue) 
		{
		}
	}

	/**
	 * Hashmap containing ImageIcons
	 */
	protected HashMap theIconDB = new HashMap( 300 );

	/**
 	 * Constructor that excepts a URL path to a jar file containing icons.
  	 * Use the getResource() method to get the ImageIcon from the Zip/Jar File.
	 * Creation date: (1/16/2001 4:07:59 PM)
 	 * @param pZipFile URL path to the jar/zip file containing the icons
 	*/
	public IconManager(URL pZipFile) 
	{
		setResourceFile( pZipFile );
	}
	
	/**
 	 * Returns the ImageIcon if it exists; null otherwise
 	 * @param pIconName the icon name
 	 * @return ImageIcon
 	 */
	public ImageIcon getResource(String pIconName) 
	{
		String convertedName = convertFilenameToUNC( pIconName );
		Object theImage = theIconDB.get( convertedName );

		if ( theImage != null ) 
		{
			return (ImageIcon)theImage;
		}
		return( this.theDefaultIcon );
	}	
	
	/**
	 * Constructor
	 * @param pBaseDirectory the parent directory of the html page containing this control.  This must be a valid URL path.
	 * @param pZipFile The path to the zip file containing image icons.  This must be a valid URL path.
	 */
	public IconManager(String pBaseDirectory, URL pZipFile) 
	{
		theBaseDirectory = pBaseDirectory;
		setResourceFile( pZipFile );
		initializeDefaultIcons();
	}
	
	/**
     * This method will add the key/data pairs to the IconDB
     * Creation date: (1/31/2001 10:15:41 AM)
     * @param pResources HashMap
     */
    public void addResources(HashMap pResources) 
    {
		Set theKeys = pResources.keySet();
		Object theTmpKey;
		Object theValue;

		for ( Iterator theIterator = theKeys.iterator(); theIterator.hasNext(); ) 
		{
			theTmpKey = theIterator.next();
			theValue = pResources.get( theTmpKey );
			theIconDB.put( theTmpKey, theValue );
		}
    }

	/**
	 * Returns the IconDB
	 * Creation date: (1/31/2001 10:41:52 AM)
	 * @return HashMap
	 */
	public HashMap getIconDB() 
	{
		return theIconDB;

	}
	
	/**
	 * Returns the number of icons in the IconDB
	 * Creation date: (1/31/2001 10:38:41 AM)
	 * @return int
	 */
	public int getNumIcons() 
	{
		return theIconDB.size();
	}
	
	/**
	 * Sets the resource file and reinitializes the IconDB with the contents
	 * of the resource file.
	 * Creation date: (1/31/2001 10:29:43 AM)
	 * @param pResourceFile URL
	 */
	public void setResourceFile(URL pResourceFile) 
	{
		if( null == theResourceFile || !theResourceFile.equals( pResourceFile ) )
		{
			theResourceFile = pResourceFile;
		}
		else
		{
			// alreadly loaded, so just return
			return;			
		}
		
		final int MAX_READ_ATTEMPTS = 20;
		try 
		{
			//open input stream connection to the file
			InputStream inputStream = pResourceFile.openStream();
			BufferedInputStream theInputStream = 
				new BufferedInputStream( inputStream );

			//create the zip input stream to the file
			ZipInputStream theZipInputStream =
				new ZipInputStream( theInputStream );

			//iterate throw the entire stream store all entries in a hashmap
			ZipEntry theZipEntry;
			int theOffset = 0;
			int theEntrySize = 0;
			int theAmountRead = 0;
			int theTotalRead = 0;
			int theNumReadAttempts = 0;
			byte[] theImageBuffer;
			byte[] theTmpBuffer;
			while( (theZipEntry = theZipInputStream.getNextEntry()) != null ) 
			{
				theEntrySize = (int)theZipEntry.getSize();
				theImageBuffer = new byte[theEntrySize];
				theTmpBuffer = new byte[theEntrySize];
				theAmountRead = 0;
				theTotalRead = 0;
				theNumReadAttempts = 0;

				while (theTotalRead < theEntrySize) 
				{
					theAmountRead = theZipInputStream.read( theTmpBuffer, theOffset, theEntrySize );

					//copy the data read to the permanent buffer
					for( int idx = 0; idx < theAmountRead; idx++ ) 
					{
						theImageBuffer[idx + theTotalRead] = theTmpBuffer[idx];
					}
					theTotalRead += theAmountRead;

					//check to see if we have read the expected number of bytes
					if ( theTotalRead >= theEntrySize ) 
					{
						theIconDB.put( theZipEntry.getName(), new ImageIcon(theImageBuffer) );
					}

					if ( theNumReadAttempts > MAX_READ_ATTEMPTS ) 
					{
						System.out.println( "Error: Unable to extract " + theZipEntry.getName() + " from " + //$NON-NLS-1$ //$NON-NLS-2$
									pResourceFile + "(Bytes Expected/Bytes Read " +  //$NON-NLS-1$
									String.valueOf(theEntrySize) + "/" + String.valueOf(theAmountRead) + ")" ); //$NON-NLS-1$ //$NON-NLS-2$
						break;
					}

					theNumReadAttempts++;	
				}	
			}

		} 
		catch (IOException ioe) 
		{
			ioe.printStackTrace();
		}
	}
	
	/**
	 * Whether the resource file is present.
	 */
	public boolean hasResourceFile()
	{
		if( null != theResourceFile )
		{
			return( true );	
		}	
		return( false );
	}
	
	/**
	 * Sets the parent directory where images can be found.
	 */
	public void setParentDirectory( String parentDirectory )
	{
		theParentDirectory = parentDirectory;	
	}
	
	/**
	 * Converts the filename to a UNC filename.
	 */
	public static String convertFilenameToUNC( String filename )
	{
		// convert any UNC separators to the system default one
		String uncSeparator = "/"; //$NON-NLS-1$
		String convertedName = filename.replace( File.separator.charAt( 0 ), 
		                                         uncSeparator.charAt( 0 ) );
		// also if file separator equals unc separator, remove any Windows
		// specific "\"
		String windowsFileSeparator = "\\"; //$NON-NLS-1$
		if( !File.separator.equals( windowsFileSeparator ) &&
		    File.separator.equals( uncSeparator ) )
		{		
			// on Unix
			convertedName = convertedName.replace( windowsFileSeparator.charAt( 0 ),
			                                       uncSeparator.charAt( 0 ) );
		}
		return( convertedName );		
	}
	
	
	
}
