package com.rational.rpw.processviewer;

import com.rational.rpw.compositetreeview.CompositeTreeView;
import com.rational.rpw.compositetreeview.TreeDropComponent;
import com.rational.rpw.dnd.datatransfer.TreeTransferable;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import java.awt.dnd.DropTargetDropEvent;
import java.util.Vector;

import java.io.File;

import com.rational.rpw.processview.*;

/**
 * This class is a drop component for process views.  Concretely,
 * it knows how to handle bookmark nodes. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class ViewerDropComponent extends TreeDropComponent
{
    /**
     * Default constructor.
     */
    public ViewerDropComponent( CompositeTreeView tree )
    {
        super( tree );
    }

    /**
     * Override to handle bookmark nodes.
     */
    protected boolean addTreeNodes( DropTargetDropEvent dropTargetDropEvent,
                                    Vector objectToDrop,
                                    MutableTreeNode parent,
                                    int row )
    {
        int dropAction = dropTargetDropEvent.getDropAction();

        Bookmark nodeToDrop = null;
		Bookmark parentBookmark = (Bookmark)parent;
        DefaultTreeModel model = (DefaultTreeModel)_tree.getModel();
		boolean isChanged = false;
        for( int i = 0; i < objectToDrop.size(); i++ )
        {
            nodeToDrop = (Bookmark)objectToDrop.elementAt( i );

            if( parentBookmark.equals( nodeToDrop ) )
            {
		        // same node, cannot drop
		        _tree.getToolkit().beep();
		        return( false );
            }
            if( parentBookmark.getParent() != null &&
                ((Bookmark)parentBookmark.getParent()).equals( nodeToDrop ) )
            {
                // descendant conflict, cannot drop
	        	_tree.getToolkit().beep();
	        	return( false );
        	}
        	if( ((Bookmark)nodeToDrop.getParent()).equals( parentBookmark ) )
        	{
                // node already in parent
	        	_tree.getToolkit().beep();
	        	return( false );        		
        	}
	        if( parent.getAllowsChildren() )
    	    {
				// do insert at end
				int rowIndex = parent.getChildCount();
				if( parent instanceof Bookmark )
				{
					// get actual index because bookmark handles visibility
	            	rowIndex = ((Bookmark)parent).getActualChildCount();
				}
	            parent.insert( (Bookmark)nodeToDrop.clone(),
	                           rowIndex );
				isChanged = true;
            }
    	    else
    	    {
	            // no children allowed, cannot drop
	            _tree.getToolkit().beep();
	            return( false );
    	    }
        }

		if( isChanged )
		{
			// notify change all at once
			model.nodeStructureChanged( parent );			
		}
		
	    return( true );
    }

    /**
     * Override to handle bookmark nodes.
     */
    protected boolean addFileList( Vector objectToDrop,
                                   MutableTreeNode parent,
                                   int row )
    {
        Bookmark bookmarkToDrop = null;
        Bookmark parentBookmark = (Bookmark)parent;
		DefaultTreeModel model = (DefaultTreeModel)_tree.getModel();
		
		// see if file is icon file
		if( parent.getParent() != null && 
		    objectToDrop.size() == 1 )
		{
			File file = (File)objectToDrop.elementAt( 0 );
			if( isIconFile( file ) )
			{
        		parentBookmark.setOpenIconName( file.getAbsolutePath() );
        		parentBookmark.setClosedIconName( file.getAbsolutePath() );
        		model.nodeStructureChanged( parentBookmark );        		
        		return( true );
			}			
		}
		
		// if other kind of file
		if( parent.getParent() != null &&
		    null == parentBookmark.getFileName() &&
            objectToDrop.size() == 1 )
        {
        	// if parent is just a bookmark, with no filename,
        	// and only one file is to be associated,
        	// then all we do here is associate the filename with
        	// the parent and set its file existence flag to true
        	parentBookmark.setFileName( ((File)objectToDrop.elementAt( 0 )).getAbsolutePath() );
        	parentBookmark.setFromContentLibrary( false );
        	parentBookmark.setExist( true );
        	model.nodeChanged( parentBookmark );
        }
        else
        {   
        	// drop new files as bookmarks into the parent   
       		boolean isChanged = false;
	        for( int i = 0; i < objectToDrop.size(); i++ )
 	       	{
        		File fileToDrop = (File)objectToDrop.elementAt( i );        	
            	bookmarkToDrop = new Bookmark( fileToDrop.getName() );
            	bookmarkToDrop.setFromContentLibrary( false );
            	bookmarkToDrop.setFileName( fileToDrop.getAbsolutePath() );
            
            	if( parentBookmark.equals( bookmarkToDrop ) )
	            {
			        // same node, cannot drop
			        _tree.getToolkit().beep();
		 	       return( false );
	            }
 	           	if( parentBookmark.getParent() != null &&
  	                ((Bookmark)parentBookmark.getParent()).equals( bookmarkToDrop ) )
	            {
    	            // descendant conflict, cannot drop
	        		_tree.getToolkit().beep();
	        		return( false );
        		}
        		if( ((Bookmark)bookmarkToDrop.getParent()) != null &&
        		    ((Bookmark)bookmarkToDrop.getParent()).equals( parentBookmark ) )
        		{
          	    	// node already in parent
	        		_tree.getToolkit().beep();
	        		return( false );        		
        		}
            	if( parent.getAllowsChildren() )
            	{
    	        	// do the insert at the end
					int rowIndex = parent.getChildCount();
					if( parent instanceof Bookmark )
					{
						// get actual index because bookmark handles visibility
		            	rowIndex = ((Bookmark)parent).getActualChildCount();
					}
	                parent.insert( bookmarkToDrop,
	                               rowIndex );
	                isChanged = true;
            	}
    	    	else
	        	{
	            	// no children allowed, cannot drop
	            	_tree.getToolkit().beep();
	            	return( false );
	        	}
 	       	}
 	       	if( isChanged )
 	       	{
 	       		// notify change all at once
 	       		model.nodeStructureChanged( parent );
 	       	}
        }

	    return( true );
    }

	/**
	 * Determine whether file is icon file.
	 */
	protected boolean isIconFile( File file )
	{
		String filename = file.getName();
		if( filename.endsWith( ".gif" ) ||
		    filename.endsWith( ".jpg" ) ||
		    filename.endsWith (".bmp" ) )
		{
       		return( true );       		
		}
		return( false );
	}
	
}
