package com.rational.rpw.processviewer;

import java.awt.Color;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DnDConstants;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;

import com.rational.rpw.compositetreeview.CompositeTreeView;
import com.rational.rpw.compositetreeview.TreePopupMenuAdapter;
import com.rational.rpw.dnd.datatransfer.FileListTransferable;
import com.rational.rpw.dnd.datatransfer.TreeTransferable;
import com.rational.rpw.environment.FontFactory;
import com.rational.rpw.processview.Bookmark;

/**
 * This class knows how to present bookmarks as trees in a tabbed panel. 
 * It knows how to add popup menus and set the trees to handle drag and
 * drop if neccessary. <br>
 * 
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class ViewerTabPanel
{
    // list of bookmarks and identifiers
    protected TreeMap _bookmarkList = null;

    // list of trees corresponding to each bookmark
    protected TreeMap _treeViewList = null;
    
    // tabbed pane
    protected JTabbedPane _tabbedPane = null;
    
	
    /**
     * Default constructor.
     */
    public ViewerTabPanel()
    {
        // init variables
        _bookmarkList = new TreeMap();
        _treeViewList = new TreeMap();
    }

    /**
     * Adds a view based on the given viewable bookmark.
     */
    public void addView( Bookmark viewableObject,
                         boolean isDragAndDropEnabled )
    {       	         
    	// adds a view with no popup menu       
		addView( viewableObject,
		         isDragAndDropEnabled,
		         false,
		         null,
		         null );
    }

    /**
     * Adds a view based on the given bookmark.  Allows
     * user to add a tree cell renderer.
     */
    public void addView( Bookmark viewableObject,
                         boolean isDragAndDropEnabled,
                         ViewerTreeCellRenderer treeCellRenderer )
    {       	         
    	// adds a view with no popup menu       
		addView( viewableObject,
		         isDragAndDropEnabled,
		         false,
		         null,
		         treeCellRenderer );
    }

    /**
     * Adds a view based on the given bookmark.  This
     * method also adds a popup menu handler.
     */
    public void addView( Bookmark viewableObject,
                         boolean isDragAndDropEnabled,
                         boolean isTreeEditable,
                         TreePopupMenuAdapter popupMenuAdapter )
    {       	                
    	// adds a view with no popup menu       
		addView( viewableObject,
		         isDragAndDropEnabled,
		         isTreeEditable,
		         popupMenuAdapter,
		         null );
        
    }

    /**
     * Adds a view based on the given bookmark.  This
     * method also adds a popup menu handler and tree cell renderer.
     */
    public void addView( Bookmark viewableObject,
                         boolean isDragAndDropEnabled,
                         boolean isTreeEditable,
                         TreePopupMenuAdapter popupMenuAdapter,
                         ViewerTreeCellRenderer treeCellRenderer )
    {       	                
        // retrieve the bookmark from the viewable object
      	createTreeView( viewableObject,
     					viewableObject.getPresentationName(),
    	                isDragAndDropEnabled,
    	                DnDConstants.ACTION_COPY_OR_MOVE,
    	                isDragAndDropEnabled,
    	                DnDConstants.ACTION_COPY_OR_MOVE,
    	                isTreeEditable,
    	                popupMenuAdapter,
    	                treeCellRenderer );        
    }

    /**
     * Adds a view based on the given bookmark. Either
     * drag and/or drop ability can be specified separately. This
     * method also adds a popup menu handler and tree cell renderer,
     * which can be null.
     */
    public void addView( Bookmark viewableObject,
                         boolean isDragEnabled,
                         int dragAction,
                         boolean isDropEnabled,
                         int dropAction,
                         boolean isTreeEditable,
                         TreePopupMenuAdapter popupMenuAdapter,
                         ViewerTreeCellRenderer treeCellRenderer )
    {       	                
        // retrieve the bookmark from the viewable object
      	createTreeView( viewableObject,
     					viewableObject.getPresentationName(),
    	                isDragEnabled,
    	                dragAction,
    	                isDropEnabled,
    	                dropAction,
    	                isTreeEditable,
    	                popupMenuAdapter,
    	                treeCellRenderer );        
    }

	/**
	 * Removes the view specified by the given bookmark name.
	 */
	public void removeView( String bookmarkName )
	{
		_bookmarkList.remove( bookmarkName );
		_treeViewList.remove( bookmarkName );
	}
	
    /**
     * Retrieves the UI panel which contains this process view.
     */
    public JComponent getViewPane()
    {   
    	JComponent component = null;
        Iterator elements = _bookmarkList.entrySet().iterator();
		if( _bookmarkList.size() == 1 &&
		    ((Bookmark)((Map.Entry)elements.next()).getValue()).getPresentationName().equals( "" ) )
		{
	        // only one with not title, just put it in a panel with no title
	        Iterator oneElement = _bookmarkList.entrySet().iterator();
	        Bookmark bookmark = (Bookmark)((Map.Entry)oneElement.next()).getValue();
			component = getViewPane( bookmark.getPresentationName(), false, null );
		}
		else
		{
			Iterator multipleElements = _bookmarkList.entrySet().iterator();
			// create the tabbed pane
			_tabbedPane = new JTabbedPane();
			_tabbedPane.setFont( FontFactory.getCommonPlainFont() );
			_tabbedPane.setBorder( BorderFactory.createEmptyBorder() );	
			while( multipleElements.hasNext() )
			{
				Bookmark bookmark = (Bookmark)((Map.Entry)multipleElements.next()).getValue();
	 	   		CompositeTreeView tree = (CompositeTreeView)_treeViewList.get( bookmark.getPresentationName() );
 	 			JScrollPane pane = new JScrollPane( tree );
 	 			_tabbedPane.add( bookmark.getPresentationName(),
 	 			                 pane );
				
			}
			component = _tabbedPane;
		}		
		return( component );
    }

    /**
     * Retrieves the UI panel for the given bookmark 
     * specified by the presentation name.
     */
    public JComponent getViewPane( String presentationName,
                                   boolean showTitle,
                                   Color titleBackground )
    {   
		return( getViewPane( presentationName,
		                      showTitle,
		                      titleBackground,
		                      null ) );
    }

    /**
     * Retrieves the UI panel for the given bookmark 
     * specified by the presentation name.
     */
    public JComponent getViewPane( String presentationName,
                                   boolean showTitle,
                                   Color titleBackground,
                                   ImageIcon icon )
    {   
		// first initialize tabbed pane
		if( null == _tabbedPane && _bookmarkList.size() > 1 )
		{
			getViewPane();
		}
	
    	JComponent component = null;
   		CompositeTreeView tree = (CompositeTreeView)_treeViewList.get( presentationName.trim() );		
		JScrollPane pane = new JScrollPane( tree );
		if( showTitle )
		{
			JTabbedPane tabbedPane = new JTabbedPane();
			tabbedPane.setBorder( BorderFactory.createEmptyBorder() );
 	 		tabbedPane.add( presentationName.trim(),
 	 	 	                pane );		
			if( null != titleBackground )
			{
				tabbedPane.setBackgroundAt( 0, titleBackground );
			}
			if( null != icon )
			{
				tabbedPane.setIconAt( 0, icon );
			}
			component = tabbedPane;
		}
		else
		{
			component = pane;
		}

		return( component );
    }

    /**
     * Adds tree selection listener to all trees in the view.
     */
    public void addTreeListener( TreeSelectionListener listener )
    {
    	Iterator trees = _treeViewList.entrySet().iterator();
        while( trees.hasNext() )
        {
        	CompositeTreeView tree = (CompositeTreeView)((Map.Entry)trees.next()).getValue();
        	// make sure it's not duplicated
        	tree.removeTreeSelectionListener( listener );
        	tree.addTreeSelectionListener( listener );
        }
    }
	
    /**
     * Adds tree model listener to all trees in the view.
     */
    public void addTreeModelListener( TreeModelListener listener )
    {
    	Iterator trees = _treeViewList.entrySet().iterator();
        while( trees.hasNext() )
        {
        	CompositeTreeView tree = (CompositeTreeView)((Map.Entry)trees.next()).getValue();
        	// make sure it's not duplicated
        	tree.getModel().removeTreeModelListener( listener );
        	tree.getModel().addTreeModelListener( listener );
        }
    }
	
    /**
     * Adds mouse listener to all trees in the view.
     */
    public void addMouseSelectionListener( MouseListener listener )
    {
    	Iterator trees = _treeViewList.entrySet().iterator();
        while( trees.hasNext() )
        {
        	CompositeTreeView tree = (CompositeTreeView)((Map.Entry)trees.next()).getValue();
        	// make sure it's not duplicated
        	tree.removeMouseListener( listener );
        	tree.addMouseListener( listener );
        }
    }

    /**
     * Adds key listener to all trees in the view.
     */
    public void addKeySelectionListener( KeyListener listener )
    {
    	Iterator trees = _treeViewList.entrySet().iterator();
        while( trees.hasNext() )
        {
        	CompositeTreeView tree = (CompositeTreeView)((Map.Entry)trees.next()).getValue();
        	// make sure it's not duplicated
        	tree.removeKeyListener( listener );
        	tree.addKeyListener( listener );            
        }
    }

	/**
	 * Adds tab change listener to the tabbed panel.
	 */
	public void addTabChangeListener( ChangeListener listener )
	{
		if( null != _tabbedPane )
		{
			// remove duplication
			_tabbedPane.removeChangeListener( listener );
			_tabbedPane.addChangeListener( listener );
		}
	}
		
	/**
	 * Retrieves the bookmark for the currently selected pane.
	 */
	public Bookmark getBookmarkForSelectedView()
	{
		Bookmark bookmark = null;
		if( null == _tabbedPane )
		{
			// only one, so just return it
   			Iterator elements = _bookmarkList.entrySet().iterator();
			bookmark = (Bookmark)((Map.Entry)elements.next()).getValue();
		}
		else
		{
			int index = _tabbedPane.getSelectedIndex();
			String identifier = _tabbedPane.getTitleAt( index );
			bookmark = (Bookmark)_bookmarkList.get( identifier );
		}
		return( bookmark );
	}
	
	/**
	 * Retrieves the bookmark given the identifier, which is
	 * the bookmark's root presentation name.
	 */
	public Bookmark getBookmark( String identifier )
	{
		Bookmark bookmark = (Bookmark)_bookmarkList.get( identifier.trim() );
		return( bookmark );
	}
	
	/**
	 * Retrieves all bookmarks.
	 */
	public Iterator getBookmarks()
	{		
		Iterator elements = _bookmarkList.entrySet().iterator();
		Vector bookmarksToReturn = new Vector();
		while( elements.hasNext() )
		{
			bookmarksToReturn.addElement( (Bookmark)((Map.Entry)elements.next()).getValue() );
		}
		return( bookmarksToReturn.iterator() );
	}
	
	/**
	 * Retrieves the currently selected index of the view.
	 * Applicable which there are multiple views (i.e. multiple
	 * bookmarks).
	 */
	public int getSelectedIndex()
	{
		int index = 0;
		if( null != _tabbedPane	)
		{
			index = _tabbedPane.getSelectedIndex();
		}
		return( index );
	}
	
	/**
	 * Retrieves the index of the bookmark.
	 * Applicable which there are multiple views (i.e. multiple
	 * bookmarks).
	 */
	public int getIndexOfBookmark( String presentatioName )
	{
		int index = 0;
		if( null == _tabbedPane )
		{
			// initialize the tabbed pane
			getViewPane();
		}
		if( null != _tabbedPane	)
		{
			for( int i = 0; i < _tabbedPane.getTabCount(); i++ )
			{
				String title = _tabbedPane.getTitleAt( i );
				if( title.equals( presentatioName.trim() ) )
				{
					index = i;
					break;
				}				
			}
		}
		return( index );
	}
	
	/**
	 * Is bookmark already in tab panel.
	 */
	public boolean isBookmarkExist( String presentatioName )
	{
		Bookmark bookmark = (Bookmark)_bookmarkList.get( presentatioName.trim() );
		boolean isExist = false;
		if( null != bookmark )
		{
			isExist = true;
		}
		return( isExist );
	}
	
	/**
	 * Sets the currently selected index of the view.
	 * Applicable which there are multiple views (i.e. multiple
	 * bookmarks).
	 */
	public void setSelectedIndex( int index )
	{
		if( _tabbedPane != null )
		{
			_tabbedPane.setSelectedIndex( index );
		}
	}
	
	public void selectTab( int index )
	{
		if( _tabbedPane != null )
		{
			int totalTabs = _tabbedPane.getTabCount();
			if (index < totalTabs)
				_tabbedPane.setSelectedIndex( index );
		}
	}
	
	/**
	 * Retrieves the currently selected bookmark in the tree view.
	 */
	public Bookmark getSelectedBookmarkInTree()
	{
		CompositeTreeView tree = getSelectedTree();
		// retrieve the selected node
		Bookmark selectedBookmark = (Bookmark)tree.getLastSelectedPathComponent();
		
		return( selectedBookmark );
	}
	
	/**
	 * Selects the particular bookmark in the tree view.
	 * @param	bookmarkToSelect	The bookmark to select.
	 */
	public void setSelectedBookmarkInTree( Bookmark bookmarkToSelect )
	{
		CompositeTreeView tree = getSelectedTree();
		// use tree to select the node
		tree.setSelectionPath( new TreePath( bookmarkToSelect.getPath() ) );		
	}
	
	/**
	 * Scrolls to the particular bookmark in the tree view.
	 * @param	bookmarkToShow	The bookmark to scroll to show.
	 */
	public void srollToBookmarkInTree( Bookmark bookmarkToShow )
	{
		CompositeTreeView tree = getSelectedTree();
		// use tree to scroll to the node
		tree.scrollPathToVisible( new TreePath( bookmarkToShow.getPath() ) );		
	}
	
	/**
	 * Determines if the bookmark is expanded.
	 */
	public boolean isBookmarkExpanded( Bookmark bookmark )
	{
		CompositeTreeView tree = getSelectedTree();
		// use tree to see if it's expanded
		return( tree.isExpanded( new TreePath( bookmark.getPath() ) ) );
	}
	
	/**
	 * Expands the bookmark.
	 */
	public void expandBookmark( Bookmark bookmarkToExpand )
	{
		CompositeTreeView tree = getSelectedTree();
		// use tree to expand bookmark
		tree.expandPath( new TreePath( bookmarkToExpand.getPath() ) );
	}
	
	/**
	 * Collapses the bookmark.
	 */
	public void collapseBookmark( Bookmark bookmarkToCollapse )
	{
		CompositeTreeView tree = getSelectedTree();
		// use tree to collapse bookmark
		tree.collapsePath( new TreePath( bookmarkToCollapse.getPath() ) );
	}
	
	/**
	 * Updates the bookmark in the tree.
	 */
	public void updateBookmark( Bookmark bookmarkToUpdate )
	{
		CompositeTreeView tree = getSelectedTree();
		// use tree to collapse bookmark
		tree.nodeStructureChanged( bookmarkToUpdate );
	}
		
	/**
	 * Gets selected tree.
	 */
	public CompositeTreeView getSelectedTree()
	{
		// get selected view first
		Bookmark currentBookmark = getBookmarkForSelectedView();
		// retrieve the tree
		CompositeTreeView tree = 
			(CompositeTreeView)_treeViewList.get( currentBookmark.getPresentationName() );
		return( tree );		
	}
	
	/**
	 * Creates tree from given bookmark.  Enable drag and drop
	 * if requested.
	 */
	protected void createTreeView( Bookmark bookmark,
				    				 String identifier,
	                                 boolean isDragEnabled,
	                                 int dragAction,
	                                 boolean isDropEnabled,
	                                 int dropAction,
	                                 boolean isTreeEditable,
	                                 TreePopupMenuAdapter popupMenuAdapter,
	                                 ViewerTreeCellRenderer treeCellRenderer )
	{
        _bookmarkList.put( identifier, bookmark );

        // create a tree view of the bookmark
        if( null == treeCellRenderer )
        {
        	treeCellRenderer = new ViewerTreeCellRenderer();
        }
        
        CompositeTreeView tree = new CompositeTreeView( bookmark,
                                                        treeCellRenderer );
		      
        // enable drag and drop
        if( isDragEnabled )
        {
            tree.enableDragSource( dragAction );        	
        }
        if( isDropEnabled )
        {
	    	DataFlavor[] flavors = new DataFlavor[]
		            		       {
                                   		TreeTransferable._treeNodeFlavor,
				                    	FileListTransferable._fileListFlavor
  		                           };
        
            tree.enableDropTarget( dropAction,
                                   flavors,
                                   new ViewerDropComponent( tree ) );
        }
        
        // enable editable
       	tree.setEditable( isTreeEditable );        
        
        // enable popup
        if( null != popupMenuAdapter )
        {
        	tree.enablePopup( popupMenuAdapter );
        }

        // add to tree view list
        _treeViewList.put( identifier, tree );

	}
}
