package com.rational.rpw.utilities;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.text.Collator;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Locale;
import java.util.StringTokenizer;
import java.util.Vector;

import com.rational.rpw.filelibrary.FileLocation;
import com.rational.rpw.filelibrary.PathMap;

/**This class contains common functions that can be used anywhere
 */
public class CommonFunctions
{
	public static final int BYTECOPY_BUFFER_SIZE = 50000;
	public static final int PATH_DEPTH_MAX = 20;
	public static final String RELATIVE_PATH = "RelativePath";
    public static final String FILE_NAME = "FileName";
	
	/**This method will delete the all files in the directory and all files in it
	 * @param theFile the file or directory
	 */
	public static void deleteFiles( File theFile ) throws IOException {
		deleteFiles(theFile, true);
	}
	
	/**
	 * the method will delete all file and directory
	 * @param theFile the absolute path to the file or directory
	 * @param deleteDirectory if true then we delete the directory after deleting the contents; otherwise just delete the contents
	 */
	public static void deleteFiles( File theFile, boolean deleteDirectory ) throws IOException {
		
		String[] theList;
		File locFile;
		
		//return if it does exist
		if ( !theFile.exists() ) {
			return;
		}
		
		//if it's just a file then delete and return
		if (theFile.isFile()) {
			if (!theFile.delete()) {
				throw new IOException( "Unable to delete \"" +
									   theFile.getAbsolutePath() );
			}
			return;
		}
				
		//the root is a directory
		theList = theFile.list();
		
		if (theList != null) {
			for( int i = 0; i < theList.length; i++ ) {
				locFile = new File( theFile.getPath() + File.separator + theList[i]);
				if ( locFile.isFile() ) {
					if ( !locFile.delete() ) {
						throw new IOException( "Unable to delete " +
												" \"" + locFile.getAbsolutePath() +
									           "\"");
					}
				}
				else {
					if ( locFile.isDirectory() ) {
						deleteFiles( locFile );
					}
				}
			}
		}
		
		boolean rtnValue = true;
		if ( theFile.isDirectory() && deleteDirectory ) {
			if ( !theFile.delete() ) {
				throw new IOException( "Unable to delete " +
											" \"" + theFile.getAbsolutePath() +
								           "\"");
			}
		}
		
		theList = null;
		locFile = null;
		
	}
	
	/**Copies the contents of one folder to another recursively - Overwrites files if they exist.
	 * @param pSource the absolute path of the source folder
	 * @param pTarget the absolute path of the target folder
	 */
	public static void FolderCopy( String pSource, String pTarget ) 
		throws IOException 
	{
		// Create a filename filter which matches everything.
		FilenameFilter theFilter = new FilenameFilter() {
					public boolean accept(File theFile, String name)
					{
						return true;	
					}
		};	
		CommonFunctions.FolderCopy( pSource, pTarget, true, theFilter );
	}
	
	public static void FolderCopy( File pSource, File pTarget, boolean overWrite ) 
		throws IOException 
	{
		// Create a filename filter which matches everything.
		FilenameFilter theFilter = new FilenameFilter() {
					public boolean accept(File theFile, String name)
					{
						return true;	
					}
		};	
		CommonFunctions.FolderCopy( pSource.getAbsolutePath(), pTarget.getAbsolutePath(), overWrite, theFilter );
	}
	
	public static void FolderCopy( File pSource, File pTarget, boolean overWrite, IProgress pMonitor ) 
		throws IOException 
	{
		// Create a filename filter which matches everything.
		FilenameFilter theFilter = new FilenameFilter() {
					public boolean accept(File theFile, String name)
					{
						return true;	
					}
		};	
		CommonFunctions.FolderCopy( pSource.getAbsolutePath(), pTarget.getAbsolutePath(), overWrite, theFilter, pMonitor );
	}
	
	
	
	/**
	 * Copies the contents of one folder to another folder recursively, except those 
	 * files which match the pattern argument.
	 * @param pSource the absolute path of the source folder.
	 * @param pTarget the absolute path of the destination folder.
	 * @param exclusion mask which indicates files that should not be copied.
	 */
	public static void FolderCopy(String pSource, 
									String pTarget,
									final String exclusionMask)
		throws IOException
	{
		// Create a filename filter using the specified exclusion mask.
		FilenameFilter theFilter = new FilenameFilter() {
					public boolean accept(File theFile, String name)
					{
						return (!name.endsWith(exclusionMask));	
					}
		};	
		CommonFunctions.FolderCopy(pSource, pTarget, true, theFilter);		
	}
	
	/**Copies the contents of one folder to another recursively
	 * @param pSource the absolute path of the source folder
	 * @param pTarget the absolute path of the target folder
	 * @param overWrite true to overwrite files if they exist
	 * @param theFilter the object which is used to selectively filter files.
	 */
	public static void FolderCopy( String pSource, 
									 String pTarget, 
									 boolean overWrite, 
									 FilenameFilter theFilter) 
		throws IOException 
	{
		FolderCopy(pSource, pTarget, overWrite, theFilter, null);
	}
	
	/**Copies the contents of one folder to another recursively
	 * @param pSource the absolute path of the source folder
	 * @param pTarget the absolute path of the target folder
	 * @param overWrite true to overwrite files if they exist
	 * @param theFilter the object which is used to selectively filter files.
	 */
	public static void FolderCopy( String pSource, 
									 String pTarget, 
									 boolean overWrite, 
									 FilenameFilter theFilter,
									 IProgress pMonitor) 
		throws IOException 
	{
		String[] theFileList;
		
		//if target equals source, then return
		if (pSource.toLowerCase().equals(pTarget.toLowerCase())) {
			return;
		}
		
		
		File theSource = new File( CommonFunctions.formatPath(pSource) );
		File theTarget = new File( CommonFunctions.formatPath(pTarget) );
		
		//check to see if the source exists or if it's not a directory
		if (!theSource.exists() ||
			!theSource.isDirectory() ) {
			throw new IOException(theSource.getAbsolutePath() + " -- " +
								  "The directory does not exist");
		}
		
		//if the target does not exist then make the directory else return if it's not a directory
		if (!theTarget.exists()) {
			if (!theTarget.mkdirs()) {
				throw new IOException(theTarget.getAbsolutePath() + " -- " +
									  "Unable to create the directory");
			}
		}
		
		// Finally process all the files which we gathered.
		theFileList = theSource.list(theFilter);	
		File locFile;
		for( int i = 0; i < theFileList.length; i++ ) {
			locFile = new File( theSource.getPath() + File.separator + theFileList[i]);
			if ( locFile.isFile() ) {
				if ( !CommonFunctions.byteFileCopy( locFile.getPath(), theTarget.getPath() + File.separator + theFileList[i], overWrite ) ) {
					throw new IOException( "Unable to copy" +
											" \"" + locFile.getPath() + "\"" );
				}
				// Update the monitor if one was passed in.
				if ( pMonitor != null )
				{
					pMonitor.increment();
				}
			}
			else {
				if ( locFile.isDirectory() ) {
					CommonFunctions.FolderCopy( locFile.getPath(), theTarget.getPath() + File.separator + theFileList[i], overWrite, theFilter, pMonitor );
				}
			}
		}
		
		theFileList = null;
		theSource = null;
		theTarget = null;
		locFile = null;
		
	}
	
	/**File copy that overwrites the target file if it exists.
	 */
	public static boolean byteFileCopy( String SourceFile, String TargetFile ) {
		return CommonFunctions.byteFileCopy( SourceFile, TargetFile, true );
	}
	
	/**File copy that overwrites the target file if it exists.
	 */
	public static boolean byteFileCopy( File SourceFile, File TargetFile ) {
		return CommonFunctions.byteFileCopy( SourceFile, TargetFile, true );
	}
	
	/**Buffered byte copy
	 */
	public static boolean byteFileCopy( String SourceFile, String TargetFile, boolean overWrite ) {
		
		File theSource = new File( SourceFile );
		File theTarget = new File( TargetFile );
		return CommonFunctions.byteFileCopy( theSource, theTarget, overWrite );
		
	}
	
	/**Buffered byte copy
	 */
	public static boolean byteFileCopy( File SourceFile, File TargetFile, boolean overWrite ) 
	{
		try {
			//make sure that the paths are using the system default for path separators
			SourceFile = CommonFunctions.formatPath( SourceFile );
			TargetFile = CommonFunctions.formatPath( TargetFile );
			
			//Check to see if it's a file
			if (!SourceFile.isFile()) {
				return false;
			}
			
			//return true if the target == source
			if (SourceFile.getAbsolutePath().toLowerCase().equals(TargetFile.getAbsolutePath().toLowerCase())) {
				return true;
			}
			
			
			File theTmpParentDir = CommonFunctions.getParent(TargetFile);
			//return false if the parent directory is null
			if (theTmpParentDir == null) {
				return false;
			}
			
			if (!theTmpParentDir.exists()) {
				if (!theTmpParentDir.mkdirs()) {
					throw new IOException( "Unable to create the directory" +
											"\"" + theTmpParentDir.getAbsolutePath() + "\"" );
				}
			}
			theTmpParentDir = null;
			
			//if the target file exists and overWrite is false we return true
			if (TargetFile.exists() && !overWrite) {
				return true;
			}
			
			byteFileCopyHelper( SourceFile.getAbsolutePath(), TargetFile.getAbsolutePath() );
			
		} catch (IOException e) {
			return false;
		} 
		
		return true;
	}
	
	private static void byteFileCopyHelper( String SourceFile, String TargetFile ) throws IOException {
		FileInputStream in = null;
		FileOutputStream out = null;
		byte[] buf = new byte[BYTECOPY_BUFFER_SIZE];
		int numBytesRead = 0;		
		try {
			in = new FileInputStream( SourceFile );
			out = new FileOutputStream( TargetFile );
			while( true) {
				//synchronized( buf ) {
					numBytesRead = in.read( buf );
					if ( numBytesRead == -1 ) {
						break;
					}
					out.write( buf, 0, numBytesRead );
				//}
			}
			
			buf = null;
		} finally {
			if ( in != null ) {
				in.close();
			}
			if ( out != null ) {
				out.close();
			}
			buf = null;
		}
	}
	
	
	/**
     * Parses the file's full pathname to obtain the relative path
     * to the content library.
     */
    public static Hashtable parseFilePath( File file )
    {
        PathMap _theMap = FileLocation.getPathMap();
        String fullPathFileName = file.getAbsolutePath();
        Hashtable pathAndName = new Hashtable();
        Iterator theIterator = _theMap.getIterator();
        while ( theIterator.hasNext() )
        {
            String libraryRoot = (String) theIterator.next();
            if( fullPathFileName.toUpperCase().startsWith( libraryRoot.toUpperCase() ) )
            {
                // the file is in the content library specified by the
                // library root, proceed to parse
                String relativePath = new String();
                String fileName = null;
                String noRootPath = fullPathFileName.substring( libraryRoot.length(),
                                                                fullPathFileName.length() );
                StringTokenizer tokens = new StringTokenizer( noRootPath, File.separator );
                while( tokens.hasMoreElements() )
                {
                    String theToken = tokens.nextToken();
                    if( tokens.hasMoreElements() )
                    {
                        // if more tokens ahead, restore this token as part of the
                        // relative path
                        relativePath = relativePath + theToken + File.separator;
                    }
                    else
                    {
                        // this is the final token, must be the file name then
                        fileName = new String( theToken );
                    }
                }
                // store in hashtable
                pathAndName.put( RELATIVE_PATH, relativePath );
                pathAndName.put( FILE_NAME, fileName );
                break;
            }
        }
        return( pathAndName );
    }
    
    /**Returns the relative link to the target file from the source file.  The parameters must be
	 * a path to a file not a directory.  This method is case-insensitive
	 * @return Relative Path
	 * @param pSource Absolute file location of the source file
	 * @param pTarget Absolute file location of the target file
	 * @param isCaseSensitive true if the method should be case sensitive
	 */
	public static String getRelativePath( String pSource, String pTarget, boolean isCaseSenitive ) {
		//return null if either parameters are null
		if ( (pSource == null) || (pTarget == null) ) {
			return null;
		}
		
		//convert to lower case
		String theSource = pSource;
		String theTarget = pTarget;
		if (!isCaseSenitive) {
			theSource = pSource.toLowerCase();
			theTarget = pTarget.toLowerCase();
		}
		
		//replace "\" with "/"
		StringBuffer theTmpBuffer = new StringBuffer( 256 );
		theTmpBuffer.append( theSource );
		for (int idx = 0; idx < theSource.length(); idx++) {
			if ( theTmpBuffer.charAt(idx) == 92 ) {
				theTmpBuffer.setCharAt( idx, '/' );
			}
		}
		theSource = theTmpBuffer.toString();
		theTmpBuffer.setLength(0);
		theTmpBuffer.append( theTarget );
		for (int idx = 0; idx < theTarget.length(); idx++) {
			if ( theTmpBuffer.charAt(idx) == 92 ) {
				theTmpBuffer.setCharAt( idx, '/' );
			}
		}
		theTarget = theTmpBuffer.toString();
		theTmpBuffer.setLength(0);
		
		//Separate Paths by "/"
		String[] srcArray = new String[PATH_DEPTH_MAX];
		String[] tarArray = new String[PATH_DEPTH_MAX];
		int position;
		int srcIdx = 0;
		int tarIdx = 0;
		
		//start with source
		while ( (position = theSource.indexOf( "/" )) != -1) {
			srcArray[srcIdx++] = theSource.substring( 0, position );
			if ( theSource.length() > ++position ) {
				theSource = theSource.substring( position );
			}
			else {
				theSource = null;
				break;
			}
		}
		if ( theSource != null ) {
			srcArray[srcIdx++] = theSource;
		}
		
		//Target Path
		while ( (position = theTarget.indexOf( "/" )) != -1) {
			tarArray[tarIdx++] = theTarget.substring( 0, position );
			if ( theTarget.length() > ++position ) {
				theTarget = theTarget.substring( position );
			}
			else {
				theTarget = null;
				break;
			}
		}
		if ( theTarget != null ) {
			tarArray[tarIdx++] = theTarget;
		}
		
		//loop until we find a difference between the two paths
		int maxIdx = Math.max( srcIdx, tarIdx );
		int idx;
		for (idx = 0; idx < maxIdx; idx++) {
			
			if ( !(srcArray[idx].equals( tarArray[idx] )) ) {
				break;
			}
			
		}
		
		//Append "../" to newPath for every remaining subdirectory in source
		for ( int tmpIdx = idx; tmpIdx < (srcIdx - 1); tmpIdx++) {
			theTmpBuffer.append( "../" );
		}
		
		//Append any remaining target path to the new path
		while ( idx < tarIdx ) {
			
			theTmpBuffer.append( tarArray[idx++] + "/" );
			
		}
		
		//remove last file separator
		if ( theTmpBuffer.length() > 0 ) {
			theTmpBuffer.setLength( theTmpBuffer.length() - 1 );
		}
		
		String theRtnValue = theTmpBuffer.toString();
		theTmpBuffer.setLength(0);
		theTmpBuffer = null;
		srcArray = null;
		tarArray = null;
		theSource = null;
		theTarget = null;
		return theRtnValue;
			
	}
		
	
	/**Returns the relative link to the target file from the source file.  The parameters must be
	 * a path to a file not a directory.  This method is case-insensitive
	 * @return Relative Path
	 * @param pSource Absolute file location of the source file
	 * @param pTarget Absolute file location of the target file
	 */
	public static String getRelativePath( String pSource, String pTarget ) {
		return getRelativePath(pSource, pTarget, false);
	}
	
	/**builds the absolute path from the current absolute path and a relative
	 * path
	 * @return absolute path
	 * @param pCurrentPath absolute path of current location
	 * @param pRelativePath relative to the target location
	 */
	public static String buildPath( String pCurrentPath, String pRelativePath ) {
			
		//return the relative path if it starts with a "file://" 
		if ( pRelativePath.startsWith( "file://" ) ) {
			return pRelativePath;
		}
		
		//If either parameters are null return null
		if ( (pCurrentPath == null) || (pRelativePath == null) ) {
			return null;
		}

		//replace "\" with "/"
		StringBuffer theTmpBuffer = new StringBuffer( 256 );
		theTmpBuffer.append( pCurrentPath );
		for (int idx = 0; idx < pCurrentPath.length(); idx++) {
			if ( theTmpBuffer.charAt(idx) == 92 ) {
				theTmpBuffer.setCharAt( idx, '/' );
			}
		}
		pCurrentPath = theTmpBuffer.toString();
		theTmpBuffer.setLength(0);
		theTmpBuffer.append( pRelativePath );
		for (int idx = 0; idx < pRelativePath.length(); idx++) {
			if ( theTmpBuffer.charAt(idx) == 92 ) {
				theTmpBuffer.setCharAt( idx, '/' );
			}
		}
		pRelativePath = theTmpBuffer.toString();
		
		//Separate Paths by "/"
		String[] srcArray = new String[PATH_DEPTH_MAX];
		String[] tarArray = new String[PATH_DEPTH_MAX];
		int position;
		int srcIdx = 0;
		int tarIdx = 0;
		
		//start with source
		while ( (position = pCurrentPath.indexOf( "/" )) != -1) {
			srcArray[srcIdx++] = pCurrentPath.substring( 0, position );
			if ( pCurrentPath.length() > ++position ) {
				pCurrentPath = pCurrentPath.substring( position );
			}
			else {
				pCurrentPath = null;
				break;
			}
		}
		
		if ( pCurrentPath != null ) {
			srcArray[srcIdx++] = pCurrentPath;
		}
		
		//Target Path
		while ( (position = pRelativePath.indexOf( "/" )) != -1) {
			tarArray[tarIdx++] = pRelativePath.substring( 0, position );
			if ( pRelativePath.length() > ++position ) {
				pRelativePath = pRelativePath.substring( position );
			}
			else {
				pRelativePath = null;
				break;
			}
		}
		if ( pRelativePath != null ) {
			tarArray[tarIdx++] = pRelativePath;
		}
		
		//count the number of ".." in the relative path
		int numSubDir = 0;
		for (int theIdx = 0; theIdx < tarIdx; theIdx++) {
			if ( tarArray[theIdx].equals("..") ) {
				numSubDir++;
			}
			else {
				break;
			}
		}
		
		//build new path
		StringBuffer theNewPath = new StringBuffer(256);
		for (int theIdx = 0; theIdx < (srcIdx - numSubDir - 1); theIdx++) {
			theNewPath.append( srcArray[theIdx] + File.separator );
		}
		
		//append remaining path of the target 
		for (int theIdx = numSubDir; theIdx < tarIdx; theIdx++) {
			theNewPath.append( tarArray[theIdx] + File.separator );
		}
		
		//remove last path separator
		theNewPath.setLength( theNewPath.length() - 1 );
		
		String theRtnValue = theNewPath.toString();
		theNewPath.setLength(0);
		theTmpBuffer.setLength(0);
		theTmpBuffer = null;
		theNewPath = null;
		srcArray = null;
		tarArray = null;
		return theRtnValue;
		
	
	}
	
	/**This method strip out all path info and return the filename
	 * @param pPath	The file path
	 * @return the file name
	 */
	public static String getFileNameFromPath( String pPath ) {

		int idx;
		String theRtnString = CommonFunctions.formatPath(pPath);
		
		idx = theRtnString.lastIndexOf(File.separator);
		if (idx != -1) {
				theRtnString = theRtnString.substring( idx + 1 );
		}
		return theRtnString;
		
	}
	
	/**
	 * String comparison method
	 * NOTE: The Collator package implements an International version of compare
	 * which does not use the strict unicode value but rather does a lexiographic
	 * sort.
	 * 
	 * @param firstArgument the first string value
	 * @param secondArgument the second string value
	 * @return Value is less than zero if source is less than target, value is zero if source and target are equal, value is greater than zero if source is greater than target. 
	 */
	static public int lexStringCompare(String firstArgument, String secondArgument) {
		Collator myCollator	= Collator.getInstance(Locale.getDefault());
		
		return myCollator.compare(firstArgument, secondArgument);		
	}
	
	/**This method takes a Vector of Strings and sorts it lexiograhpicly
	 * @param pArray the vector of String values
	 * @param return true if successful, false otherwise(the original string will be return unchanged)
	*/
	static public boolean sortStringValues(Vector pArray) {
		
		Vector theSortedArray = new Vector(pArray.size());
		
		try {
			//copy the vector
			Enumeration theElements = pArray.elements();
			while (theElements.hasMoreElements()) {
				theSortedArray.addElement(theElements.nextElement());
			}
			
			stringMergeSort(theSortedArray, 0, theSortedArray.size() - 1);
		} catch (ClassCastException cce) {
			return false;
		}
		
		pArray = theSortedArray;
		return true;
		
	}
	
	/**MergeSort of Strings
	 */
	static private void stringMergeSort(Vector pArray, int pLowerBound, int pHigherBound) throws ClassCastException {
		
		if ( pLowerBound < pHigherBound ) {
			int theMidPoint = (int)(Math.floor( (pHigherBound+pLowerBound) / 2 ));
			stringMergeSort(pArray, pLowerBound, theMidPoint);
			stringMergeSort(pArray, theMidPoint+1, pHigherBound);
			stringMergeSortHelper(pArray, pLowerBound, theMidPoint, pHigherBound);
		}
		
	}
	
	static private void stringMergeSortHelper(Vector pArray, 
											  int pLowerBound, 
											  int pMidPoint, 
											  int pHigherBound) throws ClassCastException {
		int theLowerIdx = pLowerBound;
		
		String theLeftValue;
		String theRightValue;
		
		while( theLowerIdx < pMidPoint ) {
			theLeftValue = (String)pArray.elementAt(theLowerIdx);
			theRightValue = (String)pArray.elementAt(pMidPoint);
			
			//sort if the right value is lower than the left value
			if ( lexStringCompare( theLeftValue, theRightValue ) > 0 ) {
				//insert the right value into the leftvalue location
				pArray.setElementAt( theRightValue, theLowerIdx );
				//Now place the left value in the appropriate place in the right array
				int tempIdx = pMidPoint + 1;
				while ( tempIdx <= pHigherBound ) {
					String theTestValue = (String)pArray.elementAt(tempIdx);
					if ( lexStringCompare(theLeftValue, theTestValue) > 0 ) {
						//Left value is larger so we move the test value down one and continue
						pArray.setElementAt(theTestValue, tempIdx - 1);
					}
					else {
						//Left value "fits" here
						pArray.setElementAt(theLeftValue, tempIdx - 1);
						break;
					}
					//increment
					if (++tempIdx > pHigherBound) {
						//we have reached the end so we place it here
						pArray.setElementAt(theLeftValue, tempIdx);
					}
				}
			}
		}
	}
					
	/**Replaces the any pTarget found in pSource with pReplacement.  This is case-sensitive
	 */	
	public static String replaceString(String pSource, String pTarget, String pReplacement) {
		
		int idx;
		String theWorkingString = pSource;
		while( (idx = theWorkingString.indexOf(pTarget)) != -1 ) {
			theWorkingString = theWorkingString.substring(0, idx) + 
							   pReplacement +
							   theWorkingString.substring( idx + pTarget.length() );
		}
		
		return theWorkingString;
	}
	
	/**Run a series of gc's and finalizations
	 */
	public static void CompleteGC() {
		try {
			System.gc();
			Thread.currentThread().sleep(100);
			System.runFinalization();
			Thread.currentThread().sleep(100);
			System.gc();
			Thread.currentThread().sleep(100);
			System.runFinalization();
			Thread.currentThread().sleep(100);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**Returns the a File object of the parent directory
	 */
	public static File getParent(File pFile) {
		
		//check to see if the parent directory of the target file exists
		File theNewFile = CommonFunctions.convertPathUsingDefaultPathSeparator(pFile);
			
		//return false if the target path is empty
		if (theNewFile == null) {
			return null;
		}
			
		theNewFile = new File( theNewFile.getParent() );
		
		return theNewFile;
	}
	
	/**Returns the a File object of the parent directory
	 */
	public static File convertPathUsingDefaultPathSeparator(File pFile) {
		
		//check to see if the parent directory of the target file exists
		String theTempPath = CommonFunctions.convertPathUsingDefaultPathSeparator(pFile.getAbsolutePath());
			
		if (theTempPath.equals("")) {
			return null;
		}
		
		File theNewFile = new File( theTempPath );
		
		return theNewFile;
	}
	
	/**Returns the a File object of the parent directory
	 */
	public static String convertPathUsingDefaultPathSeparator(String pFile) {
		
		//check to see if the parent directory of the target file exists
		String theTempPath = "";
		if ( File.separator.equals("\\") ) {
			theTempPath = CommonFunctions.replaceString( pFile, "/", "\\" );
		}
		else {
			theTempPath = CommonFunctions.replaceString( pFile, "\\", "/" );
		}
			
		return theTempPath;
	}
		
	/**converts the path to use system defaults, remove any trailing and leading path separators, and
	 * removes any double path separators
	 */
	public static String formatPath(String pFile) {
		//convert to default system path separators
		String thePath = CommonFunctions.convertPathUsingDefaultPathSeparator( pFile );
		
		//remove any trailing path separators
		while ( thePath.endsWith(File.separator) ) {
			thePath = thePath.substring(0, thePath.length() - 1);
		}
			
		//If we are in windows we want to keep UNC paths
		String theUNCPrefix = "";
		String theOS = System.getProperty("os.name").toUpperCase();
		if( (theOS.indexOf("WINDOWS") != -1) ) {
			if ( (thePath.startsWith("\\\\")) && (thePath.indexOf(2) != '\\') ) {
				theUNCPrefix = "\\\\";
			}
		}
		
		//Remove any double path separators
		String theDoubleChecker = File.separator + File.separator;
		while ( thePath.indexOf(theDoubleChecker) != -1 ) {
			thePath = CommonFunctions.replaceString( thePath, theDoubleChecker, File.separator );
		}
		
		thePath = theUNCPrefix + thePath;
		return thePath;
	}
	
	/**converts the path to use system defaults, remove any trailing and leading path separators, and
	 * removes any double path separators
	 */
	public static File formatPath(File pFile) {
		String thePath = pFile.getAbsolutePath();
		thePath = CommonFunctions.formatPath(thePath);
		File theNewFile = new File(thePath);
		return theNewFile;
	}
	
	/**formats a link to by URL compliant
	 */
	public static String formatLink(String pLink) {
		
		//convert all spaces to %20
		String theNewPath = CommonFunctions.replaceString( pLink, " ", "%20" );
		return theNewPath;
	}
	
	/**
	 * This method converts a file specification such as 
	 * 		d:\program files\blah\blah\blah.txt
	 * to
	 * 		d:/program files/blah/blah/blah.txt
	 */
	public static String convertToFileURL(String path)
	{
		return path.replace('\\', '/');
	}
	
	/**
	 * Compares two path for equality...This takes into account if the OS is case sensitive or not
	 * @param pPath1 the first path
	 * @param pPath2 the second path
	 * @return boolean true or false
	 */
	public static boolean comparePaths(String pPath1, String pPath2) {
		String theOS = System.getProperty("os.name").toUpperCase();
		String theVersion = System.getProperty("os.version");
		int value = 0;
		
		try {
			Double version = new Double(theVersion);
			value = version.intValue();
		} catch ( NumberFormatException nfe ) {
			// Ignore as this just means the version information from the OS is garbage.
		}
		
		// Check if the OS is some variant of windows.
		if ( (theOS.indexOf("WINDOWS") != -1 ) )
		{
            /*/It's NT
            if ( value < 5 )
            {
            }
            // All other Windows Platforms
            else
            {
            }*/
            //do a case-insensitive match
            pPath1 = pPath1.toLowerCase();
            pPath2 = pPath2.toLowerCase();
		} 
		// Else it's an unix version 
		else {
			//do nothing
		}
		
		if (pPath1.equals(pPath2)) {
           	return true;
        }
        return false;
	}
	
	/**
	 * Converts a string to its Unicode-encoded (\udddd) equivalent
	 * @param pString the string to convert
	 * @return the transformed string
	 */
	public static String encodeToUnicodeNotation(String pString) {
		StringBuffer theReturnValue = new StringBuffer();
		
		for (int idx=0; idx < pString.length(); idx++) {
			char theChar = pString.charAt(idx);
			int theIntValue = (int)theChar;
			//if the value is greater than 127 then we convert otherwise we leave it alone
			if (theIntValue > 127) {
				theReturnValue.append("\\u");
				String theHexCode = Integer.toHexString(theIntValue).toLowerCase();
				//now we append one "0" for to make sure the HexCode has four digits
				for (int idx2 = theHexCode.length(); idx2 < 4; idx2++) {
					theReturnValue.append("0");
				}
				theReturnValue.append(theHexCode);
			}
			else {
				theReturnValue.append(theChar);
			}
		}	
		return theReturnValue.toString();
	}
	
	public static void main(String[] args) {
		String theTestString="Hi";
		System.out.println("Unicode=>" + CommonFunctions.encodeToUnicodeNotation(theTestString));
	}
		
}
